import pygame
import style
import settings


class Board:
    def __init__(self, square_size, sidebar_width):
        self.square_size = square_size
        self.sidebar_width = sidebar_width
        self.board_width = square_size * 8
        self.board_height = square_size * 8
        self.total_size = (self.board_width + self.sidebar_width, self.board_height)
        self.light_color = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][0]  # Jasny kolor
        self.dark_color = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][1] 
        self.font = pygame.font.Font(style.MONTSERRAT_BOLD, int(square_size * 0.3))
        self.rows = 8
        self.cols = 8

    def draw(self, canvas, player_color="white", offset_x=0):
        for row in range(self.rows):
            for col in range(self.cols):
                # Wybór koloru pola
                color = self.light_color if (row + col) % 2 == 0 else self.dark_color
                if player_color == "white":
                    draw_row = row
                    draw_col = col
                else:
                    draw_row = self.rows - 1 - row
                    draw_col = self.cols - 1 - col

                # Rysowanie pola
                rect = pygame.Rect(
                    offset_x + draw_col * self.square_size,
                    draw_row * self.square_size,
                    self.square_size,
                    self.square_size
                )
                pygame.draw.rect(canvas, color, rect)

        # Rysowanie numerów (rang) po lewej stronie
        for row in range(self.rows):
            if player_color == "white":
                label = str(8 - row)
                display_row = row
                # Kolor tekstu zależny od tła pola (pierwsza kolumna: col=0)
                text_color = self.dark_color if (row + 0) % 2 == 0 else self.light_color
            else:
                label = str(row + 1)
                display_row = row
                # Kolor tekstu zależny od tła pola (pierwsza kolumna: col=0 dla czarnego obrócona)
                text_color = self.dark_color if (row + 0) % 2 == 0 else self.light_color

            number_text = self.font.render(label, True, text_color)
            number_rect = number_text.get_rect(topleft=(offset_x + 5, display_row * self.square_size + 5))
            canvas.blit(number_text, number_rect)

        # Rysowanie liter (files) na dole
        for col in range(self.cols):
            if player_color == "white":
                label = chr(97 + col)
                display_col = col
                # Kolor tekstu zależny od tła pola (ostatni rząd: row=7 dla białego)
                text_color = self.dark_color if (7 + col) % 2 == 0 else self.light_color
            else:
                label = chr(97 + (self.cols - 1 - col))
                display_col = col
                # Kolor tekstu zależny od tła pola (ostatni rząd: row=0 dla czarnego)
                text_color = self.light_color if (0 + col) % 2 == 0 else self.dark_color

            letter_text = self.font.render(label, True, text_color)
            letter_rect = letter_text.get_rect(
                bottomleft=(offset_x + display_col * self.square_size + self.square_size - letter_text.get_width() - 5, 
                            self.board_height - 5)
            )
            canvas.blit(letter_text, letter_rect)

        # Rysowanie ramki wokół szachownicy
        border_rect = pygame.Rect(offset_x, 0, self.board_width, self.board_height)
        pygame.draw.rect(canvas, (0, 0, 0), border_rect, 2)