import os ,sys, subprocess
import pygame
from initial_window import InitialWindow
from chess_game import ChessGame
try:
    import psutil
except ImportError:
    print("Moduł psutil nie jest dostępny. Zainstaluj go za pomocą 'pip install psutil'.")


def main():
    initial_window = InitialWindow()
    result = initial_window.run()
    if result is None:
        pygame.quit()
        sys.exit()

    mode, sock, opponent_socket, nerd_mode, allow_undo, hints_enabled, allow_unlimited_undo = result
    if mode:
        game = ChessGame(mode=mode, sock=sock, opponent_socket=opponent_socket,
                         nerd_mode=nerd_mode, allow_undo=allow_undo, hints_enabled=hints_enabled,
                         allow_unlimited_undo=allow_unlimited_undo)
    else:
        pygame.quit()
        sys.exit()

    while game.running:
        game.process_events()
        game.update()
        if not game.undo_request_pending:
            if game.turn == "white":
                game.white_time -= game.clock.get_time() / 1000
                if game.white_time <= 0:
                    game.game_over = True
                    game.game_over_message = "Czas Białych się skończył! Czarne wygrywają!"
            else:
                game.black_time -= game.clock.get_time() / 1000
                if game.black_time <= 0:
                    game.game_over = True
                    game.game_over_message = "Czas Czarnych się skończył! Białe wygrywają!"
        game.render()
        pygame.display.flip()
        game.clock.tick(60)

        if game.return_to_menu:
            game.running = False
            mainmenu_path = os.path.normpath(os.path.join(os.path.dirname(__file__), "..", "mainmenu.py"))
            if os.path.exists(mainmenu_path):
                try:
                    pygame.quit()
                    sys.exit(0)
                except subprocess.CalledProcessError as e:
                    print(f"Błąd uruchamiania mainmenu.py: {e}")
                    pygame.quit()
                    sys.exit(1)
            else:
                print(f"Błąd: Nie znaleziono mainmenu.py w {mainmenu_path}")
                pygame.quit()
                sys.exit(1)

    pygame.quit()
    sys.exit()

if __name__ == "__main__":
    main()