import pygame
from piece import get_piece_at
from config import move_sound
import socket, threading, json
from utils import pos_from_alg

def network_listener(game):
    socket_to_use = game.opponent_socket if game.mode == "host" else game.sock
    socket_to_use.settimeout(10)
    while game.running and not game.connection_lost:
        try:
            data = socket_to_use.recv(1024).decode().strip()
            if not data:
                game.connection_lost = True
                game.game_over = True
                game.game_over_message = "Utracono połączenie! Przeciwnik się rozłączył."
                cleanup_sockets(game)
                break
            if data == "SURRENDER":
                game.game_over = True
                game.game_over_message = "Przeciwnik się poddał! Wygrywasz!"
            elif data == "DRAW_REQUEST":
                game.draw_request_pending = True
                game.draw_requester = "opponent"
            elif data == "DRAW_ACCEPT":
                game.game_over = True
                game.game_over_message = "Remis zaakceptowany!"
                game.draw_request_pending = False
                game.draw_requester = None
            elif data == "DRAW_REJECT":
                game.draw_request_pending = False
                game.draw_requester = None
            elif data == "UNDO_REQUEST":
                if game.move_history:
                    game.undo_request_pending = True
                    game.undo_requester = "opponent"
                else:
                    socket_to_use.send("UNDO_REJECT\n".encode())
            elif data == "UNDO_ACCEPT":
                if game.move_history:
                    game.undo_move()
                    game.undo_request_pending = False
                    game.undo_requester = None
                    game.undo_response = "accept"
                else:
                    game.undo_request_pending = False
                    game.undo_requester = None
                    game.undo_response = "accept"
            elif data == "UNDO_REJECT":
                game.undo_request_pending = False
                game.undo_requester = None
                game.undo_response = "reject"
            elif "{" in data:
                move_data = json.loads(data)
                move_notation = move_data["move"]
                game.en_passant_info = move_data.get("en_passant")
                promotion = move_data.get("promotion")
                piece_id = move_data.get("piece_id")
                is_en_passant = move_data.get("is_en_passant", False)
                captured_pos = move_data.get("captured_pos")
                orig_pos, new_pos = move_notation.split("-")
                orig_pos = pos_from_alg(orig_pos)
                new_pos = pos_from_alg(new_pos)
                piece = next((p for p in game.pieces if p.id == piece_id), None)
                if piece:
                    reaction_time = pygame.time.get_ticks() - game.turn_start_time
                    if game.turn == "white":
                        game.white_reaction_time = reaction_time
                    else:
                        game.black_reaction_time = reaction_time
                    game.move_history.append(([p.clone() for p in game.pieces], game.en_passant_info, game.turn, move_notation))
                    game.animating = True
                    game.animating_piece = piece
                    game.anim_start_pos = orig_pos
                    game.anim_end_pos = new_pos
                    game.anim_start_time = pygame.time.get_ticks()
                    game.pieces, game.en_passant_info = game.engine.simulate_move(piece, new_pos, game.pieces, game.en_passant_info)
                    if is_en_passant and captured_pos:
                        captured_pos = (captured_pos[0], captured_pos[1])
                        target = next((p for p in game.pieces if p.pos == captured_pos and p.type == "pawn"), None)
                        if target:
                            game.pieces.remove(target)
                    moved_piece = next((p for p in game.pieces if p.id == piece_id), None)
                    if moved_piece and promotion and promotion != "none":
                        promotion_map = {
                            "queen": "hetman",
                            "rook": "wieza",
                            "bishop": "goniec",
                            "knight": "kon"
                        }
                        promotion_type = promotion_map.get(promotion, promotion)
                        moved_piece.type = promotion
                        piece_color = "Biala" if moved_piece.color == "white" else "Czarna"
                        image_key = f"{promotion_type}{piece_color}"
                        if image_key in game.chess_pieces:
                            moved_piece.image = game.chess_pieces[image_key]
                    game.turn = "black" if game.turn == "white" else "white"
                    game.turn_start_time = pygame.time.get_ticks()
                    game.update_position_count()
                    pygame.mixer.Sound.play(move_sound)
                    game.update_hint()
                    if game.nerd_mode:
                        game.last_move_time = pygame.time.get_ticks()
        except socket.timeout:
            continue
        except Exception:
            continue

def cleanup_sockets(game):
    if game.sock:
        try:
            game.sock.close()
        except:
            pass
        game.sock = None
    if game.opponent_socket:
        try:
            game.opponent_socket.close()
        except:
            pass
        game.opponent_socket = None
    game.connected = False

def cleanup_sockets(game):
    if game.sock:
        try:
            game.sock.close()
        except:
            pass
        game.sock = None
    if game.opponent_socket:
        try:
            game.opponent_socket.close()
        except:
            pass
        game.opponent_socket = None
    game.connected = False