import pygame
from utils import load_image

def load_chess_pieces(square_size):
    pieces = {}
    pieces["pionekBialy"] = load_image("pieces/classic/pionekB.png", square_size)
    pieces["pionekCzarny"] = load_image("pieces/classic/pionekC.png", square_size)
    pieces["wiezaBiala"] = load_image("pieces/classic/wiezaB.png", square_size)
    pieces["wiezaCzarna"] = load_image("pieces/classic/wiezaC.png", square_size)
    pieces["konBialy"] = load_image("pieces/classic/konB.png", square_size)
    pieces["konCzarny"] = load_image("pieces/classic/konC.png", square_size)
    pieces["goniecBialy"] = load_image("pieces/classic/goniecB.png", square_size)
    pieces["goniecCzarny"] = load_image("pieces/classic/goniecC.png", square_size)
    pieces["hetmanBialy"] = load_image("pieces/classic/hetmanB.png", square_size)
    pieces["hetmanCzarny"] = load_image("pieces/classic/hetmanC.png", square_size)
    pieces["krolBialy"] = load_image("pieces/classic/krolB.png", square_size)
    pieces["krolCzarny"] = load_image("pieces/classic/krolC.png", square_size)
    return pieces

class Piece:
    def __init__(self, id, type, color, pos, image):
        self.id = id
        self.type = type
        self.color = color
        self.pos = pos
        self.image = image
        self.has_moved = False

    def clone(self):
        new_piece = Piece(self.id, self.type, self.color, self.pos, self.image)
        new_piece.has_moved = self.has_moved
        return new_piece

    def draw(self, canvas, square_size, left_offset=0, player_color="white"):
        if self.image:
            col, row = self.pos
            if player_color == "black":
                col = 7 - col
                row = 7 - row
            image_rect = self.image.get_rect()
            offset_x = (square_size - image_rect.width) // 2
            offset_y = (square_size - image_rect.height) // 2
            x = left_offset + col * square_size + offset_x
            y = row * square_size + offset_y
            canvas.blit(self.image, (x, y))

def get_piece_at(pos, pieces_list):
    for piece in pieces_list:
        if piece.pos == pos:
            return piece
    return None
def undo_move(self):
    if self.move_history:
        self.pieces, self.en_passant_info, self.turn, move_notation = self.move_history.pop()
        current_position = self.get_position_key()
        self.position_counts[current_position] = self.position_counts.get(current_position, 0) - 1
        if self.position_counts[current_position] <= 0:
            del self.position_counts[current_position]
        self.selected_piece = None
        self.animating = False
        self.game_over = False
        pieces_positions = sorted([(p.type, p.color, p.pos) for p in self.pieces])
        print(
            f"{self.mode.capitalize()}: Move undone - Turn: {self.turn}, Move: {move_notation}, History length: {len(self.move_history)}, Pieces: {pieces_positions}")
    else:
        print(f"{self.mode.capitalize()}: No moves to undo in history!")