import os, pygame, sys

HOME_DIR = os.path.expanduser("~")
CONFIG_FILE = os.path.join(HOME_DIR, ".azbest_chess")

def asset_path(filename):
    current_dir = os.path.dirname(os.path.abspath(__file__))
    assets_dir = os.path.join(current_dir, "..", "assets")
    assets_dir = os.path.normpath(assets_dir)
    return os.path.join(assets_dir, filename)

def load_sound_from_config():
    default_sound = "pionek2.mp3"
    if os.path.exists(CONFIG_FILE):
        with open(CONFIG_FILE, "r", encoding="utf-8") as file:
            lines = file.readlines()
            if len(lines) >= 7:
                sound_name = lines[6].strip()
                if sound_name in ["pionek.mp3", "pionek2.mp3"]:
                    sound_path = asset_path(sound_name)
                    if os.path.exists(sound_path):
                        return pygame.mixer.Sound(sound_path)
    sound_path = asset_path("audio/" + sound_name)
    if os.path.exists(sound_path):
        return pygame.mixer.Sound(sound_path)
    else:
        raise FileNotFoundError(f"Nie można znaleźć pliku dźwiękowego: {sound_path}")

def pos_to_alg(pos):
    col, row = pos
    letter = chr(ord('a') + int(col))
    rank = str(8 - int(row))
    return letter + rank

def pos_from_alg(alg):
    letter, rank = alg[0], alg[1]
    col = ord(letter) - ord('a')
    row = 8 - int(rank)
    return (col, row)

def load_image(filename, square_size, scale_factor=0.9):
    path = asset_path(filename)
    if not os.path.exists(path):
        print(f"Błąd:Nie znaleziono pliku obrazu: {path}")
        return None
    try:
        img = pygame.image.load(path)
        new_size = (int(square_size * scale_factor), int(square_size * scale_factor))
        return pygame.transform.scale(img, new_size)
    except pygame.error as e:
        print(f"Nie można załadować obrazu {path}: {e}")
        return None

def is_inside(col, row):
    return 0 <= col < 8 and 0 <= row < 8