import pygame
from utils import is_inside

class Piece:
    def __init__(self, id, type, color, pos, image):
        self.id = id
        self.type = type
        self.color = color
        self.pos = pos
        self.image = image
        self.has_moved = False

    def clone(self):
        new_piece = Piece(self.id, self.type, self.color, self.pos, self.image)
        new_piece.has_moved = self.has_moved
        return new_piece

    def draw(self, canvas, square_size, display_pos=None):
        if self.image:
            pos = display_pos if display_pos else self.pos
            image_rect = self.image.get_rect()
            offset_x = (square_size - image_rect.width) // 2
            offset_y = (square_size - image_rect.height) // 2
            x = pos[0] * square_size + offset_x
            y = pos[1] * square_size + offset_y
            canvas.blit(self.image, (x, y))

    def get_attack_squares(self, pieces_list):
        attacks = []
        col, row = self.pos

        if self.type == "pawn":
            direction = -1 if self.color == "white" else 1
            for dc in [-1, 1]:
                diag = (col + dc, row + direction)
                if is_inside(*diag):
                    attacks.append(diag)

        elif self.type == "knight":
            knight_moves = [(2, 1), (2, -1), (-2, 1), (-2, -1),
                            (1, 2), (1, -2), (-1, 2), (-1, -2)]
            for dx, dy in knight_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    attacks.append(new_pos)

        elif self.type in ["bishop", "rook", "queen"]:
            directions = []
            if self.type in ["rook", "queen"]:
                directions += [(1, 0), (-1, 0), (0, 1), (0, -1)]
            if self.type in ["bishop", "queen"]:
                directions += [(1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in directions:
                new_col, new_row = col, row
                while True:
                    new_col += dx
                    new_row += dy
                    if not is_inside(new_col, new_row):
                        break
                    attacks.append((new_col, new_row))
                    if any(p.pos == (new_col, new_row) for p in pieces_list):
                        break

        elif self.type == "king":
            king_moves = [(1, 0), (-1, 0), (0, 1), (0, -1),
                          (1, 1), (1, -1), (-1, 1), (-1, -1)]
            for dx, dy in king_moves:
                new_pos = (col + dx, row + dy)
                if is_inside(*new_pos):
                    attacks.append(new_pos)

        return attacks

def get_piece_at(pos, pieces_list):
    for piece in pieces_list:
        if piece.pos == pos:
            return piece
    return None