import os
import pygame
import re
def asset_path(filename):
    return os.path.join("assets", filename)

def load_image(filename, size, scale_factor=0.9):
        image = pygame.image.load(asset_path(filename)).convert_alpha()
        new_size = (int(size * scale_factor), int(size * scale_factor))
        return pygame.transform.smoothscale(image, new_size)

def is_inside(col, row):
    return 0 <= col <= 7 and 0 <= row <= 7


def load_pgn_files(directory):
    pgn_games = []
    try:
        full_path = os.path.join(os.path.dirname(__file__), directory)
        for filename in os.listdir(full_path):
            if filename.endswith(".pgn"):
                with open(os.path.join(full_path, filename), "r") as f:
                    game_moves = []
                    for line in f:
                        if not line.startswith("[") and line.strip():
                            # Podziel linię na tokeny i odfiltruj numery ruchów (np. "1.", "2.")
                            tokens = line.strip().split()
                            for token in tokens:
                                # Pomijamy tokeny, które są numerami ruchów (np. "1.", "2.")
                                if not re.match(r'^\d+\.$', token):
                                    game_moves.append(token)
                    pgn_games.append(game_moves)
    except FileNotFoundError:
        print(f"Folder {directory} nie istnieje, zwracam pustą listę")
    except Exception as e:
        print(f"Błąd podczas ładowania PGN: {e}, zwracam pustą listę")
    return pgn_games

def pos_to_alg(pos):
    col, row = pos
    if not (0 <= col < 8 and 0 <= row < 8):
        return None
    return chr(ord('a') + col) + str(8 - row)

def pos_from_alg(alg):
    if len(alg) != 2 or not alg[0].isalpha() or not alg[1].isdigit():
        return None
    col = ord(alg[0].lower()) - ord('a')
    row = 8 - int(alg[1])
    if not (0 <= col < 8 and 0 <= row < 8):
        return None
    return (col, row)

def alg_to_pos(alg):
    """
    Konwertuje notację algebraiczną (np. 'e4') na współrzędne (kolumna, wiersz).
    Kolumny: a=0, b=1, ..., h=7
    Wiersze: 1=7, 2=6, ..., 8=0 (odwrócona numeracja dla białych na dole)
    """
    if len(alg) < 2 or alg[0] not in "abcdefgh" or alg[1] not in "12345678":
        return None
    col = ord(alg[0]) - ord("a")  # a=0, b=1, ..., h=7
    row = 8 - int(alg[1])         # 1=7, 2=6, ..., 8=0
    return (col, row)