import pygame
import settings
import style

# Inicjalizacja Pygame
pygame.init()

# Minimalny rozmiar okna
MIN_WIDTH = 800
MIN_HEIGHT = 700

# Ustawienia z modułu settings
WIDTH, HEIGHT = settings.WIDTH, settings.HEIGHT
initial_width = max(WIDTH, MIN_WIDTH)
initial_height = max(HEIGHT, MIN_HEIGHT)
SCREEN = pygame.display.set_mode((initial_width, initial_height), pygame.RESIZABLE)

# Kolory
background_color = settings.background_color if hasattr(settings, 'background_color') else (240, 240, 240)
WHITE = (255, 255, 255)
GRAY = (200, 200, 200)
DARK_GRAY = (80, 80, 80)
BLACK = (0, 0, 0)
BLACK2 = (50, 50, 50)
BLUE = (100, 149, 237)
CHESS_BROWN = (181, 136, 99)
CHESS_WHITE = (240, 217, 181)
HOVER_COLOR = settings.HOVER_COLOR

# Fonty
TITLE_FONT = pygame.font.Font(style.MONTSERRAT_BOLD, 60)
BUTTON_FONT = pygame.font.Font(style.MONTSERRAT_BOLD, 30)
INFO_FONT = pygame.font.Font(style.MONTSERRAT_MEDIUM, 30)
NAME_FONT = pygame.font.Font(style.MONTSERRAT_LIGHT, 40)

def draw_button(screen, text, x, y, w, h, color, hover_color):
    mouse = pygame.mouse.get_pos()
    click = pygame.mouse.get_pressed()
    rect = pygame.Rect(x, y, w, h)

    if rect.collidepoint(mouse):
        pygame.draw.rect(screen, hover_color, rect, border_radius=10)
        if click[0]:
            pygame.time.delay(80)
            return "main_menu"
    else:
        pygame.draw.rect(screen, color, rect, border_radius=10)

    pygame.draw.rect(screen, WHITE, rect, 2, border_radius=10)

    text_surf = BUTTON_FONT.render(text, True, WHITE)
    text_rect = text_surf.get_rect(center=(x + w // 2, y + h // 2))
    screen.blit(text_surf, text_rect)
    return None

def draw_text(screen, text, x, y, font, color=BLACK, hover=False, bg_color=None, center=False):
    text_surface = font.render(text, True, color)
    if center:
        text_rect = text_surface.get_rect(center=(x, y))
    else:
        text_rect = text_surface.get_rect(topleft=(x, y))

    if bg_color:
        bg_rect = pygame.Rect(text_rect.x - 10, text_rect.y - 5, text_rect.width + 20, text_rect.height + 10)
        pygame.draw.rect(screen, bg_color, bg_rect, border_radius=5)

    if hover and text_rect.collidepoint(pygame.mouse.get_pos()):
        hover_rect = pygame.Rect(text_rect.x - 10, text_rect.y - 5, text_rect.width + 20, text_rect.height + 10)
        pygame.draw.rect(screen, HOVER_COLOR, hover_rect, border_radius=5)

    screen.blit(text_surface, text_rect)

def main():
    global SCREEN
    pygame.display.set_caption("AUTORZY - Motorola Science Cup 2025")
    clock = pygame.time.Clock()
    running = True
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                exit()
            elif event.type == pygame.VIDEORESIZE:
                new_width = max(event.w, MIN_WIDTH)
                new_height = max(event.h, MIN_HEIGHT)
                SCREEN = pygame.display.set_mode((new_width, new_height), pygame.RESIZABLE)

        current_width, current_height = SCREEN.get_size()

        # Rysowanie szachownicy
        settings.draw_chessboard(SCREEN, current_width, current_height)

        # Dynamiczny kontener – box z autorami
        container_width = int(current_width * 0.5)
        container_height = int(current_height * 0.6)
        container_x = (current_width - container_width) // 2
        container_y = (current_height - container_height) // 2

        pygame.draw.rect(SCREEN, DARK_GRAY, (container_x, container_y, container_width, container_height), border_radius=10)
        pygame.draw.rect(SCREEN, WHITE, (container_x, container_y, container_width, container_height), 2, border_radius=10)

        # Tytuł "AUTORZY" z TITLE_FONT
        draw_text(SCREEN, "AUTORZY", container_x + container_width // 2, container_y + int(container_height * 0.12),
                  TITLE_FONT, color=WHITE, center=True)

        # Lista autorów z NAME_FONT – rozmieszczona równomiernie
        authors = [
            "Adrian Kasza",
            "Daniel Ziemba",
            "Filip Matusik",
            "Igor Zębala",
            "Kamil Sroga"
        ]
        start_y = container_y + int(container_height * 0.25)
        spacing = int(container_height * 0.12)

        mouse_pos = pygame.mouse.get_pos()
        for i, name in enumerate(authors):
            y_pos = start_y + i * spacing
            hover = pygame.Rect(container_x + 20, y_pos - 10, container_width - 40, spacing).collidepoint(mouse_pos)
            draw_text(SCREEN, name, container_x + container_width // 2, y_pos + 30, NAME_FONT,
                      color=WHITE, hover=hover, center=True)

        # Odstęp między boxem a podpisem szkoły – zwiększony do 5% wysokości okna
        school_spacing = int(current_height * 0.05)
        # Podpis szkoły z INFO_FONT i dynamicznym pozycjonowaniem
        school_text = "Zespół Szkół Nr 1 im. św. Rafała Kalinowskiego"
        school_y = container_y + container_height + school_spacing
        chess_dark = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][1]  # Ciemniejszy kolor z aktualnego zestawu
        draw_text(SCREEN, school_text, current_width // 2, school_y, BUTTON_FONT, color=WHITE,
                  center=True, bg_color=chess_dark)

        # Przycisk "Powrót" – używa NAME_FONT (już zdefiniowane w draw_button)
        button_width = int(current_width * 0.3)
        button_height = int(current_height * 0.08)
        button_x = (current_width - button_width) // 2
        button_y = school_y + int(current_height * 0.05)
        button = draw_button(SCREEN, "Powrót", button_x, button_y, button_width, button_height, DARK_GRAY,
                             HOVER_COLOR)
        if button == "main_menu":
            return "main_menu"

        pygame.display.flip()
        clock.tick(60)

if __name__ == "__main__":
    main()