import os
import sys
from utils import load_image

def resource_path(relative_path):
    """Zwraca poprawną ścieżkę do zasobu w zależności od tego, czy gra jest spakowana do EXE."""
    try:
        # Jeśli gra jest spakowana, użyj katalogu tymczasowego PyInstaller
        base_path = sys._MEIPASS
    except AttributeError:
        # W trybie developerskim użyj ścieżki względem lokalizacji pliku
        base_path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(base_path, relative_path)

def get_chess_style():
    HOME_DIR = os.path.expanduser("~")
    CONFIG_FILE = os.path.join(HOME_DIR, ".azbest_chess")

    if os.path.exists(CONFIG_FILE):
        with open(CONFIG_FILE, "r") as file:
            lines = file.readlines()
            if len(lines) >= 4:
                style = lines[3].strip()
                if style in ["Classic", "Modern", "Fantasy"]:
                    return style
    return "Classic"

def load_chess_pieces(square_size):
    style = get_chess_style()
    folder = "classic" if style == "Classic" else "modern" if style == "Modern" else "fantasy"
    
    pieces = {}
    piece_names = [
        "pionekB", "pionekC", "wiezaB", "wiezaC", 
        "konB", "konC", "goniecB", "goniecC", 
        "hetmanB", "hetmanC", "krolB", "krolC"
    ]
    
    for piece_name in piece_names:
        # Używamy resource_path do generowania poprawnej ścieżki
        piece_file = resource_path(os.path.join("assets", "pieces", folder, f"{piece_name}.png"))
        pieces[piece_name] = load_image(piece_file, square_size)
    
    return pieces