import pygame
import mainmenu
import settings
import chooseOnlineMode as COM
import sys
import style
import subprocess
import os
import importlib.util
from arcymistrz.utils import load_pgn_files  # Import z głównego utils.py

# Funkcja do uzyskania poprawnej ścieżki zasobów
def resource_path(relative_path):
    """Zwraca poprawną ścieżkę do zasobu w zależności od tego, czy gra jest spakowana do EXE."""
    if hasattr(sys, '_MEIPASS'):
        # Jeśli gra jest spakowana, użyj katalogu tymczasowego PyInstaller
        return os.path.join(sys._MEIPASS, relative_path)
    else:
        # W trybie developerskim użyj ścieżki względnej
        return os.path.join(os.path.abspath(os.path.dirname(__file__)), relative_path)

pygame.init()

def load_game_class(folder, module_name):
    folder_path = resource_path(folder)  # Użyj resource_path zamiast os.path
    if not os.path.exists(folder_path):
        raise FileNotFoundError(f"Nie znaleziono folderu: {folder_path}")
    if folder_path not in sys.path:
        sys.path.append(folder_path)
    try:
        module = importlib.import_module(module_name)
        return module
    except ImportError as e:
        raise ImportError(f"Błąd importu modułu {module_name} z folderu {folder}: {e}")

print("Current directory:", resource_path(""))

WHITE = (255, 255, 255)
GRAY = (200, 200, 200)
DARK_GRAY = (80, 80, 80)
BLACK = (0, 0, 0)
HOVER_COLOR = settings.HOVER_COLOR

FONT = pygame.font.Font(resource_path(style.MONTSERRAT_BOLD), 30)  # Poprawiona ścieżka do fontu
SCREEN = pygame.display.set_mode((settings.WIDTH, settings.HEIGHT), pygame.RESIZABLE)

# Ładowanie obrazów z użyciem resource_path
img_pvp_lan = pygame.image.load(resource_path('assets/gamemodes/PvPLANpic.png'))
img_pvp = pygame.image.load(resource_path('assets/gamemodes/PvPpic.png'))
img_pvc = pygame.image.load(resource_path('assets/gamemodes/PvC.png'))
img_pvmg = pygame.image.load(resource_path('assets/gamemodes/PvGM.png'))

def draw_button_no_click(SCREEN, text, x, y, w, h, color, hover_color):
    mouse = pygame.mouse.get_pos()
    rect = pygame.Rect(x, y, w, h)
    if rect.collidepoint(mouse):
        pygame.draw.rect(SCREEN, hover_color, rect, border_radius=10)
    else:
        pygame.draw.rect(SCREEN, color, rect, border_radius=10)
    pygame.draw.rect(SCREEN, WHITE, rect, 2, border_radius=10)
    text_surf = FONT.render(text, True, WHITE)
    text_rect = text_surf.get_rect(center=(x + w // 2, y + h // 2))
    SCREEN.blit(text_surf, text_rect)

def main(SCREEN):
    global FONT, img_pvp_lan_scaled, img_pvp_scaled, img_pvc_scaled, img_pvmg_scaled
    clock = pygame.time.Clock()
    running = True

    while running:
        chess_dark = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][1]

        current_width = SCREEN.get_width()
        current_height = SCREEN.get_height()
        scale_x = current_width / settings.WIDTH
        scale_y = current_height / settings.HEIGHT

        img_pvp_lan_scaled = pygame.transform.smoothscale(img_pvp_lan, (int(250 * scale_x), int(200 * scale_y)))
        img_pvp_scaled = pygame.transform.smoothscale(img_pvp, (int(250 * scale_x), int(200 * scale_y)))
        img_pvc_scaled = pygame.transform.smoothscale(img_pvc, (int(250 * scale_x), int(200 * scale_y)))
        img_pvmg_scaled = pygame.transform.smoothscale(img_pvmg, (int(250 * scale_x), int(200 * scale_y)))

        container_width = int(270 * scale_x)
        container_height = int(470 * scale_y)
        offset_x = int(45 * scale_x)
        box1_x = 35 * scale_x
        box2_x = box1_x + container_width + offset_x
        box3_x = box2_x + container_width + offset_x
        box4_x = box3_x + container_width + offset_x
        top_y = (current_height // 2) - int(270 * scale_y)

        box1_rect = pygame.Rect(box1_x, top_y, container_width, container_height)
        box2_rect = pygame.Rect(box2_x, top_y, container_width, container_height)
        box3_rect = pygame.Rect(box3_x, top_y, container_width, container_height)
        box4_rect = pygame.Rect(box4_x, top_y, container_width, container_height)

        return_button_width = int(200 * scale_x)
        return_button_height = int(50 * scale_y)
        return_button_x = current_width // 2 - return_button_width // 2
        return_button_y = current_height // 2 + int(285 * scale_y)
        return_button_rect = pygame.Rect(return_button_x, return_button_y, return_button_width, return_button_height)

        settings.draw_chessboard(SCREEN, current_width, current_height)

        TEXT_FONT = pygame.font.Font(resource_path(style.MONTSERRAT_BOLD), int(20 * min(scale_x, scale_y)))  # Poprawiona ścieżka
        DESC_FONT = pygame.font.Font(resource_path(style.MONTSERRAT_SEMI_BOLD), int(15 * min(scale_x, scale_y)))  # Poprawiona ścieżka

        mouse_pos = pygame.mouse.get_pos()

        for box_rect, img, title, description in [
            (box1_rect, img_pvp_lan_scaled, "ROZGRYWKA W SIECI", "Zagraj z przyjacielem przez sieć lokalną lub przez internet."),
            (box2_rect, img_pvp_scaled, "GRACZ VS GRACZ", "Klasyczna lub błyskawiczna rozgrywka dla dwóch graczy na jednym urządzeniu. Zmierz się ze znajomym na jednym komputerze!"),
            (box3_rect, img_pvc_scaled, "GRACZ VS KOMPUTER", "Sprawdź swoje umiejętności w pojedynku przeciwko sztucznej inteligencji. Idealny tryb do nauki i doskonalenia strategii."),
            (box4_rect, img_pvmg_scaled, "GRACZ VS ARCYMISTRZ", "Zmierz się z wirtualnym arcymistrzem szachowym. Najwyższy poziom wyzwania dla doświadczonych graczy!")
        ]:
            if box_rect.collidepoint(mouse_pos):
                hover_color = tuple(max(0, c - 20) for c in chess_dark)
                pygame.draw.rect(SCREEN, hover_color, box_rect, border_radius=10)
            else:
                pygame.draw.rect(SCREEN, chess_dark, box_rect, border_radius=10)
            pygame.draw.rect(SCREEN, DARK_GRAY, box_rect, 2, border_radius=10)

            image_x = box_rect.x + (box_rect.width - img.get_width()) // 2
            image_y = box_rect.y + int(20 * scale_y)
            SCREEN.blit(img, (image_x, image_y))

            title_surf = TEXT_FONT.render(title, True, WHITE)
            title_rect = title_surf.get_rect(center=(box_rect.centerx, (box_rect.y + 6) + container_height // 2))
            SCREEN.blit(title_surf, title_rect)

            desc_lines = []
            words = description.split()
            line = ""
            for word in words:
                test_line = line + " " + word if line else word
                test_surf = DESC_FONT.render(test_line, True, WHITE)
                if test_surf.get_width() < box_rect.width - 20:
                    line = test_line
                else:
                    desc_lines.append(line)
                    line = word
            if line:
                desc_lines.append(line)

            desc_start_y = title_rect.bottom + int(2 * scale_y)
            for i, desc_line in enumerate(desc_lines):
                desc_surf = DESC_FONT.render(desc_line, True, WHITE)
                desc_rect = desc_surf.get_rect(center=(box_rect.centerx, desc_start_y + i * int(30 * scale_y) + 30))
                SCREEN.blit(desc_surf, desc_rect)

        draw_button_no_click(SCREEN, "Powrót", return_button_rect.x, return_button_rect.y,
                            return_button_rect.width, return_button_rect.height,
                            DARK_GRAY, HOVER_COLOR)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == pygame.VIDEORESIZE:
                new_width = max(event.w, 600)
                new_height = max(event.h, 500)
                SCREEN = pygame.display.set_mode((new_width, new_height), pygame.RESIZABLE)
            elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                pygame.time.delay(150)
                if box1_rect.collidepoint(mouse_pos):
                    COM.mode_selection_screen()
                elif box2_rect.collidepoint(mouse_pos):
                    try:
                        obj_game_module = load_game_class("pvp", "game")
                        Game = getattr(obj_game_module, "Game")
                        print(f"Loaded Game: {Game}")
                        game = Game(SCREEN)
                        result = game.run()
                        if result == "quit":
                            return "quit"
                        elif result == "main_menu":
                            continue
                    except Exception as e:
                        print(f"Error starting PvP mode: {e}")
                        continue
                elif box3_rect.collidepoint(mouse_pos):
                    try:
                        obj_game_module = load_game_class("obj", "game")
                        ChessGame = getattr(obj_game_module, "ChessGame")
                        print(f"Loaded ChessGame: {ChessGame}")
                        game = ChessGame(SCREEN)
                        result = game.run()
                        if result == "quit":
                            return "quit"
                        elif result == "main_menu":
                            continue
                    except Exception as e:
                        print(f"Error starting PvC mode: {e}")
                        continue
                elif box4_rect.collidepoint(mouse_pos):
                    try:
                        pgn_games = load_pgn_files(resource_path("games"))  # Poprawiona ścieżka do folderu games
                        print(f"Loaded pgn_games for Grandmaster mode: {pgn_games}")
                        arcymistrz_game_module = load_game_class("arcymistrz", "game")
                        ChessGame = getattr(arcymistrz_game_module, "ChessGame")
                        print(f"Loaded ChessGame: {ChessGame}")
                        game = ChessGame(SCREEN, pgn_games)
                        result = game.run()
                        if result == "quit":
                            return "quit"
                        elif result == "main_menu":
                            continue
                    except Exception as e:
                        import traceback
                        print(f"Error starting Grandmaster mode: {e}")
                        traceback.print_exc()
                        continue
                elif return_button_rect.collidepoint(mouse_pos):
                    pygame.time.delay(80)
                    return "main_menu"

        pygame.display.flip()
        clock.tick(60)

    return "quit"

if __name__ == "__main__":
    result = main(SCREEN)
    if result == "main_menu":
        mainmenu.main(SCREEN)
    pygame.quit()