import pygame
import sys
import settings

# Inicjalizacja Pygame
pygame.init()

import gamemodes
import ranking
import authors
import login
import os
import style

def resource_path(relative_path):
    """Zwraca ścieżkę do zasobu, działającą w skrypcie i pliku .exe"""
    if getattr(sys, 'frozen', False):  # Jeśli aplikacja jest spakowana
        base_path = sys._MEIPASS  # Katalog tymczasowy PyInstaller
    else:
        base_path = os.path.abspath(".")  # Bieżący katalog roboczy
    return os.path.join(base_path, relative_path)

# Załaduj ikonę z dynamiczną ścieżką
icon_path = resource_path('assets/logo.png')
icon = pygame.image.load(icon_path)
pygame.display.set_icon(icon)

HOME_DIR = os.path.expanduser("~")
CONFIG_FILE = os.path.join(HOME_DIR, ".azbest_chess")

# Kolory
WHITE = (255, 255, 255)
GRAY = (200, 200, 200)
DARK_GRAY = (80, 80, 80)
BLACK = (0, 0, 0)
BLACK2 = (40, 40, 40)
BLUE = (100, 149, 237)
CHESS_BROWN = (181, 136, 99)
CHESS_WHITE = (240, 217, 181)
HOVER_COLOR = settings.HOVER_COLOR

# Czcionka
FONT = pygame.font.Font(style.MONTSERRAT_BOLD, 30)

# Ustawienia z modułu settings
WIDTH, HEIGHT = settings.WIDTH, settings.HEIGHT
SCREEN = pygame.display.set_mode((WIDTH, HEIGHT), pygame.RESIZABLE)
pygame.display.set_caption("SZACHY AZBESTEST - Motorola Science Cup 2025")

# Funkcja rysowania przycisku
def draw_button(screen, text, x, y, w, h, color, hover_color):
    mouse = pygame.mouse.get_pos()
    click = pygame.mouse.get_pressed()
    rect = pygame.Rect(x, y, w, h)
    if rect.collidepoint(mouse):
        pygame.draw.rect(screen, hover_color, rect, border_radius=10)
        if click[0]:
            pygame.time.delay(150)
            return True
    else:
        pygame.draw.rect(screen, color, rect, border_radius=10)
    pygame.draw.rect(screen, WHITE, rect, 2, border_radius=10)
    text_surf = FONT.render(text, True, WHITE)
    text_rect = text_surf.get_rect(center=(x + w // 2, y + h // 2))
    screen.blit(text_surf, text_rect)
    return False

def main(SCREEN):
    clock = pygame.time.Clock()
    running = True
    
    pygame.mixer.music.load(settings.resource_path('assets/audio/ChessMusicTest.mp3'))
    pygame.mixer.music.play(loops=-1, fade_ms=0)
    
    while running:
        pygame.display.update()
        pygame.display.set_caption("SZACHY AZBESTEST - Motorola Science Cup 2025")
        settings.load_settings()
        settings.draw_chessboard(SCREEN, SCREEN.get_width(), SCREEN.get_height())

        current_width, current_height = SCREEN.get_size()

        # Skalowanie przycisków
        scale_x = current_width / WIDTH
        scale_y = current_height / HEIGHT
        button_width = int(400 * scale_x)  # Poszerzone z 330 do 400
        button_height = int(80 * scale_y)  # Wysokość bez zmian

        # Obliczanie pozycji z odstępem
        center_x = (current_width - button_width) // 2
        button_spacing = 30  # Odstęp między przyciskami
        total_buttons_height = 3 * button_height + 2 * button_spacing  # Wysokość wszystkich przycisków + odstępy
        center_y = ((current_height - total_buttons_height) // 2) - 50  # Początkowa pozycja y

        # Rysowanie przycisków z większym odstępem
        if draw_button(SCREEN, "Graj", center_x, center_y, button_width, button_height, DARK_GRAY, HOVER_COLOR):
            pygame.event.clear()  # Usunięcie zdarzeń myszy
            gamemodes.main(SCREEN)
        if draw_button(SCREEN, "Ranking", center_x, center_y + button_height + button_spacing, button_width, button_height, DARK_GRAY, HOVER_COLOR):
            pygame.event.clear()  # Usunięcie zdarzeń myszy
            ranking.main(SCREEN)
        if draw_button(SCREEN, "Ustawienia", center_x, center_y + 2 * (button_height + button_spacing), button_width, button_height, DARK_GRAY, HOVER_COLOR):
            pygame.event.clear()  # Usunięcie zdarzeń myszy
            settings.main()
        if draw_button(SCREEN, "Wyjdź", center_x, center_y + 3 * (button_height + button_spacing), button_width, button_height, DARK_GRAY, HOVER_COLOR):
            running = False
        
        # Przyciski w rogach (bez zmian szerokości)
        if draw_button(SCREEN, "Autorzy", 20, current_height - 70, 200, 50, DARK_GRAY, HOVER_COLOR):
            pygame.event.clear()  # Usunięcie zdarzeń myszy
            authors.main()
        if draw_button(SCREEN, "Konto", current_width - 220, 20, 200, 50, DARK_GRAY, HOVER_COLOR):
            pygame.event.clear()  # Usunięcie zdarzeń myszy
            login.main(SCREEN)

        # Obsługa zdarzeń
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            elif event.type == pygame.VIDEORESIZE:
                new_width = max(event.w, 800)
                new_height = max(event.h, 700)
                SCREEN = pygame.display.set_mode((new_width, new_height), pygame.RESIZABLE)
        
        clock.tick(60)
        
    pygame.quit()

if __name__ == "__main__":
    main(SCREEN)