import pygame
import sys
import os
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
import settings
import style

class Board:
    def __init__(self, square_size=80, rows=8, cols=8, sidebar_width=300, player_color="white"):
        self.rows = rows
        self.cols = cols
        self.square_size = square_size
        self.board_width = self.cols * self.square_size
        self.board_height = self.rows * self.square_size
        self.player_color = player_color
        
        self.light_color = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][0]
        self.dark_color = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][1]
        
        self.sidebar_width = sidebar_width
        self.total_size = (self.board_width + self.sidebar_width, self.board_height)
        self.font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size / 3))

    def draw(self, canvas, player_color="white"):
        # Rysowanie pól planszy z odpowiednią transformacją współrzędnych
        for row in range(self.rows):
            for col in range(self.cols):
                if player_color == "white":
                    display_row = self.rows - 1 - row
                    display_col = col
                else:
                    display_row = row
                    display_col = self.cols - 1 - col

                color = self.light_color if (row + col) % 2 == 0 else self.dark_color
                rect = pygame.Rect(display_col * self.square_size,
                                display_row * self.square_size,
                                self.square_size, self.square_size)
                pygame.draw.rect(canvas, color, rect)

        # Rysowanie obramowania planszy
        border_rect = pygame.Rect(0, 0, self.board_width, self.board_height)
        pygame.draw.rect(canvas, (0, 0, 0), border_rect, 2)

        # Rysowanie numerów (rang) po lewej stronie planszy
        for i in range(self.rows):
            if player_color == "white":
                label = str(self.rows - i)
                display_y = i
                text_color = self.light_color if (i + 0) % 2 == 0 else self.dark_color
            else:
                label = str(i + 1)
                display_y = i
                text_color = self.light_color if (i + 0) % 2 == 0 else self.dark_color

            text_surface = self.font.render(label, True, text_color)
            canvas.blit(text_surface, (5, display_y * self.square_size + 5))

        # Rysowanie liter (files) na dolnej krawędzi planszy
        for j in range(self.cols):
            if player_color == "white":
                label = chr(97 + j)
                display_x = j
                text_color = self.light_color if (7 + j) % 2 == 0 else self.dark_color
            else:
                label = chr(97 + (self.cols - 1 - j))
                display_x = j
                text_color = self.dark_color if (0 + j) % 2 == 0 else self.light_color

            text_surface = self.font.render(label, True, text_color)
            pos_x = display_x * self.square_size + self.square_size - text_surface.get_width() - 5
            pos_y = self.board_height - text_surface.get_height() - 5
            canvas.blit(text_surface, (pos_x, pos_y))

if __name__ == "__main__":
    pygame.init()
    screen = pygame.display.set_mode((800, 600))
    pygame.display.set_caption("Szachownica")
    board = Board()
    running = True
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
        screen.fill((255, 255, 255))
        board.draw(screen)
        pygame.display.flip()
    pygame.quit()