import pygame
import style
class Board:
    def __init__(self, square_size=80, rows=8, cols=8,
                 light_color=(240, 217, 181), dark_color=(181, 136, 99),
                 sidebar_width=300):
        self.rows = rows
        self.cols = cols
        self.square_size = square_size
        self.board_width = self.cols * self.square_size
        self.board_height = self.rows * self.square_size
        self.light_color = light_color
        self.dark_color = dark_color
        self.sidebar_width = sidebar_width
        self.total_size = (self.board_width + self.sidebar_width, self.board_height)
        self.font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size / 3))
        self.player_color = "white"

    def draw(self, canvas, player_color="white"):
        # Draw squares (still adjust for player perspective)
        for row in range(self.rows):
            for col in range(self.cols):
                adjusted_col = 7 - col if player_color == "black" else col
                adjusted_row = 7 - row if player_color == "black" else row
                color = self.light_color if (row + col) % 2 == 0 else self.dark_color
                rect = pygame.Rect(adjusted_col * self.square_size, adjusted_row * self.square_size,
                                 self.square_size, self.square_size)
                pygame.draw.rect(canvas, color, rect)

        # Draw border
        border_rect = pygame.Rect(0, 0, self.board_width, self.board_height)
        pygame.draw.rect(canvas, (0, 0, 0), border_rect, 2)

        CHESS_BROWN = (181, 136, 99)
        CHESS_WHITE = (240, 217, 181)

        # Draw row numbers (always 1 at bottom, 8 at top)
        for row in range(self.rows):
            number = str(8 - row)  # Always show 1 at bottom
            text_color = CHESS_BROWN if row % 2 == 0 else CHESS_WHITE
            left_text = self.font.render(number, True, text_color)
            left_rect = left_text.get_rect(topleft=(5, row * self.square_size + 5))
            canvas.blit(left_text, left_rect)

        # Draw column letters (always a at left, h at right)
        for col in range(self.cols):
            letter = chr(97 + col)  # Always show a at left
            text_color = CHESS_WHITE if col % 2 == 0 else CHESS_BROWN
            bottom_text = self.font.render(letter, True, text_color)
            bottom_rect = bottom_text.get_rect(bottomright=(col * self.square_size + self.square_size - 5,
                                              self.board_height - 5))
            canvas.blit(bottom_text, bottom_rect)
