import pygame
import sys
import os
import websocket
import json
import threading
import queue
import time
import socket
import style

from utils import *
from engine import *
from board import *
from piece import *
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from chess_assets import load_chess_pieces, get_chess_style
import settings
import style
pygame.init()
pygame.mixer.init()

pygame.display.set_caption("SZACHY ONLINE - Motorola Science Cup 2025")

CONFIG_FILE = ""
HOME_DIR = os.path.expanduser("~")
if len(sys.argv) > 1:
    CONFIG_FILE = sys.argv[1]
else:
    CONFIG_FILE = os.path.join(HOME_DIR, ".azbest_chess")

DOMAIN = "azbest.org.pl"

def resource_path(relative_path):
    """Zwraca poprawną ścieżkę do zasobu w zależności od tego, czy gra jest spakowana do EXE."""
    try:
        # Jeśli gra jest spakowana, użyj katalogu tymczasowego PyInstaller
        base_path = sys._MEIPASS
    except AttributeError:
        # W trybie developerskim użyj ścieżki względem lokalizacji pliku
        base_path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(base_path, relative_path)

# Inicjalizacja ustawień
settings.load_settings()

# Ładowanie dźwięku ruchu z poprawną ścieżką
move_sound = pygame.mixer.Sound(resource_path(os.path.join("assets/audio", settings.CURRENT_SOUND)))
pygame.mixer.Sound.set_volume(move_sound, (settings.sfprev_value) / 100)

class ChessGame:
    def __init__(self, room_id, access_token, nerd_mode=False,  hints_enabled=False, api_request = None):
            pygame.init()
            pygame.font.init()

            self.room_id = room_id

            # Dodaj w __init__ po innych przyciskach
            self.button_rect_accept = None
            self.button_rect_reject = None

            # Game state variables
            self.draw_request_pending = False
            self.draw_requester = None
            self.draw_response = None
            # Use separate fonts: bold for titles, regular for values
            self.font = pygame.font.Font(style.MONTSERRAT_BOLD, 36)
            self.small_font = pygame.font.Font(style.MONTSERRAT_REGULAR, 24)  # Regular font
            self.bold_small_font = pygame.font.Font(style.MONTSERRAT_BOLD, 24)  # Bold font for nerd labels

            # Board setup
            square_size = 80
            sidebar_width = 300
            self.board = Board(square_size=square_size, sidebar_width=sidebar_width)
            self.total_size = self.board.total_size
            self.canvas = pygame.display.set_mode(self.total_size)
            self.square_size = self.board.square_size

            self.api_request = api_request

            # Load chess pieces
            self.chess_pieces = load_chess_pieces(self.square_size)
            self.pionekBialy = self.chess_pieces["pionekB"]
            self.pionekCzarny = self.chess_pieces["pionekC"]
            self.wiezaBiala = self.chess_pieces["wiezaB"]
            self.wiezaCzarna = self.chess_pieces["wiezaC"]
            self.konBialy = self.chess_pieces["konB"]
            self.konCzarny = self.chess_pieces["konC"]
            self.goniecBialy = self.chess_pieces["goniecB"]
            self.goniecCzarny = self.chess_pieces["goniecC"]
            self.hetmanBialy = self.chess_pieces["hetmanB"]
            self.hetmanCzarny = self.chess_pieces["hetmanC"]
            self.krolBialy = self.chess_pieces["krolB"]
            self.krolCzarny = self.chess_pieces["krolC"]

            # Game variables
            self.pieces = []
            self.piece_id_counter = 0
            self.turn = "white"
            self.selected_piece = None
            self.en_passant_info = None
            self.move_history = []
            self.position_counts = {}
            self.game_over = False
            self.game_over_message = ""
            self.clock = pygame.time.Clock()
            self.animating = False
            self.animating_piece = None
            self.anim_start_pos = None
            self.anim_end_pos = None
            self.anim_start_time = 0
            self.anim_duration = 500
            self.return_to_menu = False
            self.promotion_pending = False
            self.promoting_pawn = None

            # Network variables
            self.room_id = room_id
            self.access_token = access_token
            self.ws = None
            self.ws_thread = None
            self.connected = False
            self.connection_lost = False
            self.message_queue = queue.Queue()
            self.player_color = None
            self.opponent_color = None
            self.last_ping_sent = None
            self.ping = None

            # Game options
            self.nerd_mode = nerd_mode
            self.hints_enabled = hints_enabled
            self.hint_pos = None
            self.undo_request_pending = False
            self.undo_requester = None
            self.undo_response = None

            # Engine and game state
            self.running = True
            self.game_started = False
            self.engine = ChessEngine()

            # Time tracking
            self.white_time = 300  # 5 minutes in seconds
            self.black_time = 300  # 5 minutes in seconds
            self.last_time_update = None

            # Sidebar ma szerokość 300 pikseli, zaczyna się od x=640, kończy na x=940
            # Przyciski miały width=200, teraz będą miały 230, więc przesuwamy x z 690 na 675, aby wyśrodkować
            self.button_rect_surrender = pygame.Rect(675, 410, 230, 40)  # x z 690 na 675, width z 200 na 230
            self.button_rect_draw = pygame.Rect(675, 460, 230, 40)       # x z 690 na 675, width z 200 na 230
            self.button_rect_undo = pygame.Rect(675, 510, 230, 40)       # x z 690 na 675, width z 200 na 230
            self.button_rect_nerd = pygame.Rect(675, 560, 230, 40)       # x z 690 na 675, width z 200 na 230

            self.button_rect_cancel = None
            self.button_rect_back = None

            # Nerd mode info
            self.fps = 0
            self.server_ip = self.resolve_domain_ip(DOMAIN)
            self.public_ip = self.fetch_public_ip()

            self.last_ping_time = 0
            self.ping_interval = 2

            self.notification_queue = []  # Queue to store pending notifications
            self.current_notification = None  # Currently displayed notification
            self.last_notification = None  # Track the last displayed notification to avoid duplicates
            self.notification_start_time = 0
            self.notification_duration = 3000  # 3 seconds display time per notification
            self.notification_height = 50
            self.notification_anim_duration = 250  # Faster animation: 250ms for slide in/out (was 500ms)
            self.notification_y_offset = 0  # For animation



            # Setup the board
            self.setup_pieces()

            # Start WebSocket connection
            self.start_websocket()

    def show_notification(self, message):
            """Add a new notification to the queue, preventing duplicates in a row."""
            # Only add if the message is different from the last one displayed
            if message != self.last_notification or not self.notification_queue:
                self.notification_queue.append(message)
                # If no notification is currently active, start displaying immediately
                if not self.current_notification:
                    self._start_next_notification()

    def _start_next_notification(self):
        """Start displaying the next notification from the queue."""
        if self.notification_queue:
            self.current_notification = self.notification_queue.pop(0)  # Dequeue the next message
            self.notification_start_time = pygame.time.get_ticks()
            self.notification_y_offset = self.notification_height
    def resolve_domain_ip(self, domain):
        try:
            # Remove port if present (e.g., "localhost:8080" -> "localhost")
            clean_domain = domain.split(':')[0]
            ip = socket.gethostbyname(clean_domain)
            return f"{domain} -> {ip}"
        except Exception as e:
            print(f"Failed to resolve domain IP: {e}")
            return domain  # Fallback to just domain if resolution fails

    def fetch_public_ip(self):
        try:
            #response = requests.get("http://icanhazip.com", timeout=5)
            return self.api_request("ip",access_token=self.access_token).text.strip()
        except Exception as e:
            print(f"Failed to fetch public IP: {e}")
            return "N/A"

    def send_ping(self):
        if self.connected and self.ws:
            try:
                self.last_ping_sent = time.time()
                self.ws.send("ping")  # Send as text message
            except Exception as e:
                print(f"Failed to send ping: {e}")

    def start_websocket(self):
        def on_message(ws, message):
            try:
                # Handle ping-pong messages
                if isinstance(message, str) and message == "pong":
                    if self.last_ping_sent:
                        self.ping = int((time.time() - self.last_ping_sent) * 1000)
                    return

                # Handle other messages
                data = json.loads(message)
                self.message_queue.put(data)
            except Exception as e:
                print(f"Failed to parse message: {e}")
                self.message_queue.put({"type": "error", "payload": {"message": str(e)}})
        def on_error(ws, error):
            print(f"WebSocket error: {error}")
            if not self.connected:
                self.return_to_menu = True
                return
            self.message_queue.put({"type": "error", "payload": {"message": str(error)}})

        def on_close(ws, close_status_code, close_msg):
            print(f"WebSocket closed: {close_status_code} - {close_msg}")
            self.connected = False
            self.message_queue.put({"type": "connection_lost"})

        def on_open(ws):
            print("WebSocket connected")
            self.connected = True
            self.message_queue.put({"type": "connected"})

        ws_url = f"ws://{DOMAIN}/rooms/{self.room_id}"
        headers = {"Authorization": f"Bearer {self.access_token}"}
        self.ws = websocket.WebSocketApp(
            ws_url,
            header=headers,
            on_open=on_open,
            on_message=on_message,
            on_error=on_error,
            on_close=on_close
        )
        self.ws_thread = threading.Thread(target=self.ws.run_forever, kwargs={"ping_interval": 5}, daemon=True)
        self.ws_thread.start()

    def send_websocket_message(self, message):
        if self.connected and self.ws:
            try:
                self.ws.send(json.dumps(message))
            except Exception as e:
                print(f"Failed to send WebSocket message: {e}")


    def process_messages(self):
        while not self.message_queue.empty():
            try:
                data = self.message_queue.get_nowait()
                self.handle_message(data)
            except queue.Empty:
                break

    def handle_message(self, data):
        msg_type = data.get("type")
        payload = data.get("payload", {})
        print(f"{msg_type} = {payload}")
        if msg_type == "color_assignment":
            self.player_color = payload["player_color"]
            self.opponent_color = "black" if self.player_color == "white" else "white"
            self.board.player_color = self.player_color
            print(f"Assigned color: {self.player_color}")
            self.render()

        elif msg_type == "move":
                move_notation = payload["move"]
                orig, dest = move_notation.split("-")
                orig_pos = pos_from_alg(orig)
                dest_pos = pos_from_alg(dest)
                piece = get_piece_at(orig_pos, self.pieces)
                if piece:
                    move_sound.play()
                    self.move_history.append(([piece.clone() for piece in self.pieces], self.en_passant_info, self.turn, move_notation))
                    self.pieces, self.en_passant_info = self.engine.simulate_move(piece, dest_pos, self.pieces, self.en_passant_info)
                    moved_piece = get_piece_at(dest_pos, self.pieces)
                    if moved_piece and moved_piece.type == "pawn" and len(move_notation) > 5:  # Promotion included
                        promotion = move_notation[5].lower()  # e.g., "Q" -> "queen"
                        if promotion == "q":
                            moved_piece.type = "queen"
                            moved_piece.image = self.hetmanBialy if moved_piece.color == "white" else self.hetmanCzarny
                        elif promotion == "r":
                            moved_piece.type = "rook"
                            moved_piece.image = self.wiezaBiala if moved_piece.color == "white" else self.wiezaCzarna
                        elif promotion == "b":
                            moved_piece.type = "bishop"
                            moved_piece.image = self.goniecBialy if moved_piece.color == "white" else self.goniecCzarny
                        elif promotion == "n":
                            moved_piece.type = "knight"
                            moved_piece.image = self.konBialy if moved_piece.color == "white" else self.konCzarny
                    elif moved_piece and moved_piece.type == "pawn":  # Fallback for old messages without promotion
                        if (moved_piece.color == "white" and dest_pos[1] == 0) or (moved_piece.color == "black" and dest_pos[1] == 7):
                            # Assume queen if no promotion specified (backward compatibility)
                            moved_piece.type = "queen"
                            moved_piece.image = self.hetmanBialy if moved_piece.color == "white" else self.hetmanCzarny
                    self.turn = "black" if self.turn == "white" else "white"
                    self.update_position_count()

        elif msg_type == "draw_request":
            self.draw_request_pending = True
            self.draw_requester = "opponent"

        elif msg_type == "draw_response":
            response = payload["accept"]
            if response:
                self.game_over = True
                self.game_over_message = "Remis zaakceptowany!"
            self.draw_request_pending = False
            self.draw_requester = None

        elif msg_type == "undo_request":
            self.undo_request_pending = True
            self.undo_requester = "opponent"

        elif msg_type == "undo_response":
            response = payload["accept"]
            if response:
                self.undo_move()
            self.undo_request_pending = False
            self.undo_requester = None

        elif msg_type == "game_state":
            self.turn = payload["turn"]

        elif msg_type == "game_over" and not self.game_over:
            reason = payload["reason"]
            self.game_over = True
            if reason == "checkmate":
                self.game_over_message = "Szach-mat! " + ("Wygrywasz!" if self.turn != self.player_color else "Przegrywasz!")
            elif reason == "stalemate":
                self.game_over_message = "Pat! Remis!"
            elif reason == "surrender":
                self.game_over_message = "Przeciwnik się poddał! Wygrywasz!"
            elif reason == "timeout":
                self.game_over_message = "Czas minął! " + ("Wygrywasz!" if self.turn != self.player_color else "Przegrywasz!")
            elif reason == "50-move rule":
                self.game_over_message = "Reguła 50 ruchów! Remis!"
            elif reason == "insufficient material":
                self.game_over_message = "Niewystarczający materiał! Remis!"

        elif msg_type == "error":
            print(f"[SERVER ERROR] {payload['message']}")

        elif msg_type == "connection_lost":
            self.connection_lost = True
            if not self.game_over:
                self.game_over = True
                self.game_over_message = "Utracono połączenie!"

        elif msg_type == "connected":
            self.connected = True

        elif msg_type == "start":
            self.game_started = True
            self.last_time_update = pygame.time.get_ticks()
        elif msg_type == "time_update":
            self.white_time = payload["white_time"]
            self.black_time = payload["black_time"]
            self.last_time_update = pygame.time.get_ticks()

    def send_move(self, move_notation):
        if self.connected:
            self.send_websocket_message({"type": "move", "payload": {"move": move_notation}})

    def send_draw_request(self):
        if self.connected:
            self.send_websocket_message({"type": "draw_request"})

    def send_draw_response(self, accept):
        if self.connected:
            self.send_websocket_message({"type": "draw_response", "payload": {"accept": accept}})

    def send_undo_request(self):
        if self.connected:
            self.send_websocket_message({"type": "undo_request"})

    def send_undo_response(self, accept):
        if self.connected:
            self.send_websocket_message({"type": "undo_response", "payload": {"accept": accept}})

    def send_surrender(self):
        if self.connected:
            self.send_websocket_message({"type": "surrender"})

    def setup_pieces(self):
        self.add_piece("rook", "white", 0, 7, self.wiezaBiala, self.wiezaCzarna)
        self.add_piece("knight", "white", 1, 7, self.konBialy, self.konCzarny)
        self.add_piece("bishop", "white", 2, 7, self.goniecBialy, self.goniecCzarny)
        self.add_piece("queen", "white", 3, 7, self.hetmanBialy, self.hetmanCzarny)
        self.add_piece("king", "white", 4, 7, self.krolBialy, self.krolCzarny)
        self.add_piece("bishop", "white", 5, 7, self.goniecBialy, self.goniecCzarny)
        self.add_piece("knight", "white", 6, 7, self.konBialy, self.konCzarny)
        self.add_piece("rook", "white", 7, 7, self.wiezaBiala, self.wiezaCzarna)
        for col in range(8):
            self.add_piece("pawn", "white", col, 6, self.pionekBialy, self.pionekCzarny)
        self.add_piece("rook", "black", 0, 0, self.wiezaBiala, self.wiezaCzarna)
        self.add_piece("knight", "black", 1, 0, self.konBialy, self.konCzarny)
        self.add_piece("bishop", "black", 2, 0, self.goniecBialy, self.goniecCzarny)
        self.add_piece("queen", "black", 3, 0, self.hetmanBialy, self.hetmanCzarny)
        self.add_piece("king", "black", 4, 0, self.krolBialy, self.krolCzarny)
        self.add_piece("bishop", "black", 5, 0, self.goniecBialy, self.goniecCzarny)
        self.add_piece("knight", "black", 6, 0, self.konBialy, self.konCzarny)
        self.add_piece("rook", "black", 7, 0, self.wiezaBiala, self.wiezaCzarna)
        for col in range(8):
            self.add_piece("pawn", "black", col, 1, self.pionekBialy, self.pionekCzarny)

    def add_piece(self, piece_type, color, col, row, image_white, image_black):
        image = image_white if color == "white" else image_black
        self.pieces.append(Piece(self.piece_id_counter, piece_type, color, (col, row), image))
        self.piece_id_counter += 1

    def get_position_key(self):
        board_state = []
        for piece in sorted(self.pieces, key=lambda p: (p.pos[0], p.pos[1])):
            board_state.append(f"{piece.type}{piece.color}{piece.pos}")
        return "|".join(board_state) + f"|{self.turn}|{self.en_passant_info}"

    def draw_pieces(self):
        for piece in self.pieces:
            if piece == self.animating_piece and self.animating:
                anim_col, anim_row = self.get_anim_pos()
                if self.player_color == "black":
                    anim_col = 7 - anim_col
                    anim_row = 7 - anim_row
                x = anim_col * self.square_size
                y = anim_row * self.square_size
                image_rect = piece.image.get_rect()
                offset_x = (self.square_size - image_rect.width) // 2
                offset_y = (self.square_size - image_rect.height) // 2
                self.canvas.blit(piece.image, (x + offset_x, y + offset_y))
            else:
                piece.draw(self.canvas, self.square_size, player_color=self.player_color if self.player_color else "white")

    def get_anim_pos(self):
        elapsed = pygame.time.get_ticks() - self.anim_start_time
        if elapsed >= self.anim_duration:
            self.animating = False
            return self.anim_end_pos
        progress = elapsed / self.anim_duration
        start_col, start_row = self.anim_start_pos
        end_col, end_row = self.anim_end_pos
        current_col = start_col + (end_col - start_col) * progress
        current_row = start_row + (end_row - start_row) * progress
        return (current_col, current_row)

    def surrender(self):
        self.game_over = True
        if self.player_color == "white":
            self.game_over_message = "Poddajesz się! Czarne wygrywają!"
        else:
            self.game_over_message = "Poddajesz się! Białe wygrywają!"
        if self.connected:
            self.send_surrender()

    def propose_draw(self):
        if self.connected and not self.draw_request_pending:
            self.draw_request_pending = True
            self.draw_requester = "player"
            self.send_draw_request()

    def undo_move(self):
        if self.move_history:
            # Log all entries in move_history before undoing
            print(f"{self.player_color}: Move history before undo: {[m[3] for m in self.move_history]}")

            # Get and remove the last move from history
            prev_pieces, prev_en_passant, prev_turn, move_notation = self.move_history.pop()
            print(f"{self.player_color}: Removing move from history: {move_notation}")

            # Store current state for comparison
            current_pieces = [piece.clone() for piece in self.pieces]

            # Restore the previous state
            self.pieces = [piece.clone() for piece in prev_pieces]
            self.en_passant_info = prev_en_passant
            self.turn = prev_turn
            current_position = self.get_position_key()
            self.position_counts[current_position] = self.position_counts.get(current_position, 0) - 1
            if self.position_counts[current_position] <= 0:
                del self.position_counts[current_position]
            self.selected_piece = None
            self.animating = False
            self.game_over = False

            # Analyze differences between current_pieces and restored self.pieces
            current_dict = {p.id: (p.type, p.color, p.pos) for p in current_pieces}
            prev_dict = {p.id: (p.type, p.color, p.pos) for p in self.pieces}

            moved = []
            removed = []
            added = []

            for pid in set(current_dict.keys()).union(prev_dict.keys()):
                if pid in current_dict and pid in prev_dict:
                    curr_type, curr_color, curr_pos = current_dict[pid]
                    prev_type, prev_color, prev_pos = prev_dict[pid]
                    if curr_pos != prev_pos:
                        moved.append(f"{curr_type.capitalize()} ({curr_color}) moved from {pos_to_alg(curr_pos)} to {pos_to_alg(prev_pos)}")
                    elif curr_type != prev_type:
                        moved.append(f"{curr_type.capitalize()} ({curr_color}) at {pos_to_alg(curr_pos)} changed to {prev_type.capitalize()}")
                elif pid in current_dict:
                    curr_type, curr_color, curr_pos = current_dict[pid]
                    removed.append(f"{curr_type.capitalize()} ({curr_color}) removed from {pos_to_alg(curr_pos)}")
                elif pid in prev_dict:
                    prev_type, prev_color, prev_pos = prev_dict[pid]
                    added.append(f"{prev_type.capitalize()} ({prev_color}) added to {pos_to_alg(prev_pos)}")

            # Log the detailed changes
            print(f"{self.player_color}: Restored state - Turn: {self.turn}, En Passant: {self.en_passant_info}")
            if moved:
                print(f"{self.player_color}: Moved: {', '.join(moved)}")
            if removed:
                print(f"{self.player_color}: Removed: {', '.join(removed)}")
            if added:
                print(f"{self.player_color}: Added: {', '.join(added)}")
            if not (moved or removed or added):
                print(f"{self.player_color}: No changes detected in piece positions.")
            print(f"{self.player_color}: Restored Pieces: {[(p.type, p.pos) for p in self.pieces]}")
            print(f"{self.player_color}: Move history after undo: {[m[3] for m in self.move_history]}")

            # Ensure visual update
            self.render()
            pygame.display.flip()
        else:
            print(f"{self.player_color}: No moves to undo!")
    def check_insufficient_material(self):
        if len(self.pieces) > 4:
            return False
        white_pieces = [p for p in self.pieces if p.color == "white"]
        black_pieces = [p for p in self.pieces if p.color == "black"]
        if len(white_pieces) == 1 and len(black_pieces) == 1:
            return True
        if len(white_pieces) == 1 and len(black_pieces) == 2:
            other_piece = next((p for p in black_pieces if p.type != "king"), None)
            return other_piece.type in ["knight", "bishop"]
        if len(black_pieces) == 1 and len(white_pieces) == 2:
            other_piece = next((p for p in white_pieces if p.type != "king"), None)
            return other_piece.type in ["knight", "bishop"]
        if len(white_pieces) == 2 and len(black_pieces) == 2:
            white_non_king = next((p for p in white_pieces if p.type != "king"), None)
            black_non_king = next((p for p in black_pieces if p.type != "king"), None)
            if white_non_king.type == "bishop" and black_non_king.type == "bishop":
                white_bishop_pos = white_non_king.pos
                black_bishop_pos = black_non_king.pos
                white_color = (white_bishop_pos[0] + white_bishop_pos[1]) % 2
                black_color = (black_bishop_pos[0] + black_bishop_pos[1]) % 2
                return white_color == black_color
        return False

    def handle_promotion(self, pawn):
        self.promotion_pending = True
        self.promoting_pawn = pawn
        promotion_options = ["queen", "rook", "bishop", "knight"]
        images = {
            "queen": self.hetmanBialy if pawn.color == "white" else self.hetmanCzarny,
            "rook": self.wiezaBiala if pawn.color == "white" else self.wiezaCzarna,
            "bishop": self.goniecBialy if pawn.color == "white" else self.goniecCzarny,
            "knight": self.konBialy if pawn.color == "white" else self.konCzarny
        }
        overlay = pygame.Surface(self.total_size)
        overlay.set_alpha(200)
        overlay.fill((100, 100, 100))
        self.canvas.blit(overlay, (0, 0))
        promotion_size = self.square_size
        total_width = promotion_size * 4
        x0 = (self.total_size[0] - total_width) // 2
        y0 = (self.total_size[1] - promotion_size) // 2
        for i, option in enumerate(promotion_options):
            rect = pygame.Rect(x0 + i * promotion_size, y0, promotion_size, promotion_size)
            pygame.draw.rect(self.canvas, (255, 255, 255), rect)
            pygame.draw.rect(self.canvas, (0, 0, 0), rect, 2)
            img = images[option]
            if img:
                img_rect = img.get_rect()
                offset_x = (promotion_size - img_rect.width) // 2
                offset_y = (promotion_size - img_rect.height) // 2
                self.canvas.blit(img, (rect.x + offset_x, rect.y + offset_y))
        pygame.display.update()

        chosen = None
        while chosen is None:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.cleanup()
                    pygame.quit()
                    sys.exit()
                elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                    pos = event.pos
                    for i, option in enumerate(promotion_options):
                        rect = pygame.Rect(x0 + i * promotion_size, y0, promotion_size, promotion_size)
                        if rect.collidepoint(pos):
                            chosen = option
                            break
        pawn.type = chosen
        pawn.image = images[chosen]
        self.promotion_pending = False
        self.promoting_pawn = None

    def update_position_count(self):
        current_position = self.get_position_key()
        self.position_counts[current_position] = self.position_counts.get(current_position, 0) + 1

    def process_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.cleanup()
                pygame.quit()
                sys.exit()

            elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                if not self.game_started:
                    mouse_pos = pygame.mouse.get_pos()
                    if self.button_rect_cancel.collidepoint(mouse_pos):
                        self.cleanup()
                        self.return_to_menu = True
                    continue
                if self.game_over:
                    mouse_pos = pygame.mouse.get_pos()
                    if self.button_rect_back.collidepoint(mouse_pos):
                        self.cleanup()
                        self.return_to_menu = True
                        return
                if self.animating or self.promotion_pending:
                    return

                mouse_pos = pygame.mouse.get_pos()

                if self.draw_request_pending and self.draw_requester != "player":
                    if self.button_rect_accept and self.button_rect_accept.collidepoint(mouse_pos):
                        self.draw_response = "accept"
                        self.game_over = True
                        self.game_over_message = "Remis zaakceptowany!"
                        if self.connected:
                            self.send_draw_response(True)
                        self.draw_request_pending = False
                        self.draw_requester = None
                        return
                    elif self.button_rect_reject and self.button_rect_reject.collidepoint(mouse_pos):
                        self.draw_response = "reject"
                        if self.connected:
                            self.send_draw_response(False)
                        self.draw_request_pending = False
                        self.draw_requester = None
                        return

                if self.undo_request_pending and self.undo_requester != "player":
                    if self.button_rect_accept and self.button_rect_accept.collidepoint(mouse_pos):
                        if self.connected:
                            self.send_undo_response(True)
                        self.undo_request_pending = False
                        self.undo_requester = None
                        return
                    elif self.button_rect_reject and self.button_rect_reject.collidepoint(mouse_pos):
                        if self.connected:
                            self.send_undo_response(False)
                        self.undo_request_pending = False
                        self.undo_requester = None
                        return

                if self.button_rect_surrender.collidepoint(mouse_pos):
                    self.surrender()
                    return

                if self.button_rect_draw.collidepoint(mouse_pos):
                    if self.connected and not self.draw_request_pending:
                        if self.player_color == self.turn:
                            self.draw_request_pending = True
                            self.draw_requester = "player"
                            self.send_draw_request()
                        else:
                            self.show_notification("Nie możesz zaproponować remisu poza swoją turą!")
                    elif self.draw_request_pending:
                        self.show_notification("Propozycja remisu już została wysłana!")
                    return

                if self.button_rect_undo.collidepoint(mouse_pos):
                    print(f"{self.player_color}: Undo button clicked - pending: {self.undo_request_pending}, turn: {self.turn}, player_color: {self.player_color}, animating: {self.animating}, game_over: {self.game_over}")
                    if not self.undo_request_pending and not self.animating and not self.game_over:
                        if not self.move_history:  # Check if there are moves to undo
                            print(f"{self.player_color}: No moves to undo")
                            self.show_notification("Brak ruchów do cofnięcia!")
                        elif self.connected:
                            if self.player_color != self.turn:
                                print(f"{self.player_color}: Sending undo request")
                                self.undo_request_pending = True
                                self.undo_requester = "player"
                                self.send_undo_request()
                            else:
                                print(f"{self.player_color}: Cannot undo - it’s your turn")
                                self.show_notification("Nie możesz cofnąć ruchu podczas swojej tury!")
                        else:
                            print(f"{self.player_color}: Cannot undo - not connected")
                            self.show_notification("Brak połączenia z serwerem!")
                    else:
                        print(f"{self.player_color}: Undo blocked - pending: {self.undo_request_pending}, animating: {self.animating}, game_over: {self.game_over}")
                        if self.undo_request_pending:
                            self.show_notification("Czekam na odpowiedź przeciwnika!")
                        elif self.animating:
                            self.show_notification("Poczekaj na zakończenie animacji!")
                        elif self.game_over:
                            self.show_notification("Gra się zakończyła!")
                    return

                if self.button_rect_nerd.collidepoint(mouse_pos):
                    self.nerd_mode = not self.nerd_mode
                    return

                if mouse_pos[0] < self.board.board_width:
                    col = (7 - (mouse_pos[0] // self.board.square_size)) if self.player_color == "black" else (
                        mouse_pos[0] // self.board.square_size)
                    row = (7 - (mouse_pos[1] // self.board.square_size)) if self.player_color == "black" else (
                        mouse_pos[1] // self.board.square_size)
                    clicked_pos = (col, row)

                    if self.connected and self.player_color is None:
                        return

                    if self.turn != self.player_color:
                        return

                    if not self.promotion_pending and not self.animating:
                        if self.selected_piece is None:
                            piece = get_piece_at(clicked_pos, self.pieces)
                            if piece and piece.color == self.turn:
                                self.selected_piece = piece
                        else:
                            target_piece = get_piece_at(clicked_pos, self.pieces)
                            if target_piece and target_piece.color == self.turn:
                                self.selected_piece = target_piece
                            else:
                                valid_moves = self.engine.get_valid_moves(self.selected_piece, self.pieces, self.en_passant_info)
                                if clicked_pos in valid_moves:
                                    orig_pos = self.selected_piece.pos
                                    move_notation = pos_to_alg(orig_pos) + "-" + pos_to_alg(clicked_pos)
                                    # Log before adding to move_history
                                    print(f"{self.player_color}: Adding local move {move_notation} to history")
                                    self.move_history.append(([piece.clone() for piece in self.pieces], self.en_passant_info, self.turn, move_notation))
                                    print(f"{self.player_color}: Move history updated: {[m[3] for m in self.move_history]}")
                                    self.animating = True
                                    self.animating_piece = self.selected_piece
                                    self.anim_start_pos = orig_pos
                                    self.anim_end_pos = clicked_pos
                                    self.anim_start_time = pygame.time.get_ticks()
                                    self.pieces, self.en_passant_info = self.engine.simulate_move(self.selected_piece, clicked_pos, self.pieces, self.en_passant_info)
                                    moved_piece = get_piece_at(clicked_pos, self.pieces)
                                    if moved_piece and moved_piece.type == "pawn":
                                        if (moved_piece.color == "white" and clicked_pos[1] == 0) or (
                                                moved_piece.color == "black" and clicked_pos[1] == 7):
                                            self.handle_promotion(moved_piece)
                                    if self.connected:
                                        self.send_move(move_notation)
                                    self.turn = "black" if self.turn == "white" else "white"
                                    self.update_position_count()
                                    move_sound.play()
                                    print(f"{self.player_color}: After local move {move_notation}, Pieces: {[(p.type, p.pos) for p in self.pieces]}")
                                self.selected_piece = None

            elif event.type == pygame.KEYDOWN and event.key == pygame.K_h and self.hints_enabled and not self.promotion_pending:
                if self.turn == self.player_color and self.selected_piece:
                    hint = self.engine.get_hint(self.selected_piece, self.pieces, self.en_passant_info)
                    if hint:
                        self.hint_pos = hint
                    else:
                        self.hint_pos = None


    def update(self):
        if self.animating:
            elapsed = pygame.time.get_ticks() - self.anim_start_time
            if elapsed >= self.anim_duration:
                self.animating = False
                self.animating_piece.pos = self.anim_end_pos

        if self.current_notification:
            elapsed = pygame.time.get_ticks() - self.notification_start_time
            if elapsed < self.notification_anim_duration:  # Slide in (faster: 250ms)
                progress = elapsed / self.notification_anim_duration
                self.notification_y_offset = self.notification_height * (1 - progress)
            elif elapsed >= self.notification_duration - self.notification_anim_duration:  # Slide out (faster: 250ms)
                remaining = (self.notification_duration - elapsed) / self.notification_anim_duration
                self.notification_y_offset = self.notification_height * (1 - remaining)
            if elapsed >= self.notification_duration:  # End current notification (after 3s total)
                self.last_notification = self.current_notification  # Store the last notification
                self.current_notification = None
                self._start_next_notification()

    def render_game_over(self):
        overlay = pygame.Surface(self.total_size, pygame.SRCALPHA)
        overlay.fill((0, 0, 0, 180))
        self.canvas.blit(overlay, (0, 0))

        game_over_text = self.font.render(self.game_over_message, True, (255, 255, 255))
        text_rect = game_over_text.get_rect(center=(self.total_size[0] // 2, self.total_size[1] // 2 - 50))
        self.canvas.blit(game_over_text, text_rect)

        mouse_pos = pygame.mouse.get_pos()
        self.button_rect_back = pygame.Rect(self.total_size[0] // 2 - 100, self.total_size[1] // 2 + 20, 200, 40)
        back_hover = self.button_rect_back.collidepoint(mouse_pos)

        if back_hover:
            pygame.draw.rect(self.canvas, (100, 100, 100), self.button_rect_back.move(3, 3), border_radius=12)
        back_color = (70, 70, 70) if back_hover else (50, 50, 50)
        pygame.draw.rect(self.canvas, back_color, self.button_rect_back, border_radius=12)
        back_text = self.small_font.render("Wyjdź", True, (255, 255, 255))
        self.canvas.blit(back_text, (
            self.button_rect_back.x + (self.button_rect_back.width - back_text.get_width()) // 2,
            self.button_rect_back.y + (self.button_rect_back.height - back_text.get_height()) // 2
        ))

    def render(self):
        self.canvas.fill((50, 50, 50))
        self.board.draw(self.canvas, self.player_color if self.player_color else "white")
        self.draw_pieces()

        if self.selected_piece:
            col, row = self.selected_piece.pos
            if self.player_color == "black":
                col = 7 - col
                row = 7 - row
            rect = pygame.Rect(col * self.square_size, row * self.square_size, self.square_size, self.square_size)
            pygame.draw.rect(self.canvas, (0, 255, 0), rect, 3)
            valid_moves = self.engine.get_valid_moves(self.selected_piece, self.pieces, self.en_passant_info)
            for move in valid_moves:
                m_col, m_row = move
                if self.player_color == "black":
                    m_col = 7 - m_col
                    m_row = 7 - m_row
                s = pygame.Surface((self.square_size, self.square_size))
                s.set_alpha(100)
                s.fill((255, 255, 0))
                self.canvas.blit(s, (m_col * self.square_size, m_row * self.square_size))

        current_king = next((p for p in self.pieces if p.type == "king" and p.color == self.turn), None)
        if current_king and self.engine.is_in_check(self.turn, self.pieces):
            col, row = current_king.pos
            if self.player_color == "black":
                col = 7 - col
                row = 7 - row
            rect = pygame.Rect(col * self.square_size, row * self.square_size, self.square_size, self.square_size)
            pygame.draw.rect(self.canvas, (255, 0, 0), rect, 3)
            check_text = self.font.render("Szach!", True, (255, 0, 0))
            self.canvas.blit(check_text, (10, 10))

        if self.hint_pos and self.hints_enabled:
            col, row = self.hint_pos
            if self.player_color == "black":
                col = 7 - col
                row = 7 - row
            s = pygame.Surface((self.square_size, self.square_size))
            s.set_alpha(150)
            s.fill((0, 255, 0))
            self.canvas.blit(s, (col * self.square_size, row * self.square_size))

        CHESS_WHITE = (240, 217, 181)
        sidebar_rect = pygame.Rect(self.board.board_width, 0, self.board.sidebar_width, self.board.board_height)
        pygame.draw.rect(self.canvas, CHESS_WHITE, sidebar_rect)

        self.white_time = max(0,self.white_time)
        self.black_time = max(0,self.black_time)

        white_mins = int(self.white_time // 60)
        white_secs = int(self.white_time % 60)
        black_mins = int(self.black_time // 60)
        black_secs = int(self.black_time % 60)

        white_color = (255, 0, 0) if self.white_time < 60 else (0, 0, 0)
        black_color = (255, 0, 0) if self.black_time < 60 else (0, 0, 0)

        white_clock_text = self.small_font.render(f"Białe: {white_mins:02d}:{white_secs:02d}", True, white_color)
        black_clock_text = self.small_font.render(f"Czarne: {black_mins:02d}:{black_secs:02d}", True, black_color)

        turn_text = self.small_font.render("Tura: " + ("Białe" if self.turn == "white" else "Czarne"), True, (0, 0, 0))
        moves_title = self.small_font.render("Historia ruchów", True, (0, 0, 0))
        self.canvas.blit(white_clock_text, (self.board.board_width + 20, 20))
        self.canvas.blit(black_clock_text, (self.board.board_width + 20, 60))
        self.canvas.blit(turn_text, (self.board.board_width + 20, 100))

        y_offset = 140
        self.canvas.blit(moves_title, (self.board.board_width + 20, y_offset))
        y_offset += 40
        for i, move in enumerate(self.move_history[-7:]):
            move_text = self.small_font.render(move[3], True, (0, 0, 0))
            self.canvas.blit(move_text, (self.board.board_width + 20, y_offset + i * 30))

        if self.nerd_mode:
                self.fps = self.clock.get_fps()

                # Define text strings (combined label and value)
                fps_text = f"FPS: {self.fps:.1f}"
                ping_text = f"PING: {self.ping if self.ping is not None else 'N/A'} ms"
                server_text = f"SERWER: {self.server_ip}"
                ip_text = f"IP: {self.public_ip}"

                # Set colors based on player role
                if self.player_color == "black":
                    text_color = (0, 0, 0)    # Black text
                    outline_color = (255, 255, 255)  # White outline
                else:  # white player or no color assigned yet
                    text_color = (255, 255, 255)  # White text
                    outline_color = (0, 0, 0)    # Black outline

                # Render bold text
                fps_surface = self.bold_small_font.render(fps_text, True, text_color)
                ping_surface = self.bold_small_font.render(ping_text, True, text_color)
                server_surface = self.bold_small_font.render(server_text, True, text_color)
                ip_surface = self.bold_small_font.render(ip_text, True, text_color)

                # Adjust the vertical spacing to fill the gap
                nerd_info = [
                    (fps_text, fps_surface, 10),    # Original position
                    (ping_text, ping_surface, 40),  # Original position
                    (server_text, server_surface, 70),  # Moved up to fill gap
                    (ip_text, ip_surface, 100)      # Moved up to fill gap
                ]

                for text_str, surface, y in nerd_info:
                    # Create outline using the text string
                    outline = self.bold_small_font.render(text_str, True, outline_color)
                    for dx, dy in [(-1, -1), (-1, 1), (1, -1), (1, 1)]:
                        self.canvas.blit(outline, (10 + dx, y + dy))
                    # Render the main text
                    self.canvas.blit(surface, (10, y))

        self.BLACK = (0, 0, 0)
        self.WHITE = (255, 255, 255)
        self.HOVER_COLOR_WHITE = (220, 220, 220)
        self.HOVER_COLOR_BLACK = (50, 50, 50)
        self.SHADOW_COLOR = (150, 150, 150)

        mouse_pos = pygame.mouse.get_pos()
        surrender_hover = self.button_rect_surrender.collidepoint(mouse_pos)
        draw_hover = self.button_rect_draw.collidepoint(mouse_pos)
        undo_hover = self.button_rect_undo.collidepoint(mouse_pos)
        nerd_hover = self.button_rect_nerd.collidepoint(mouse_pos)

        if self.player_color == "white":
            default_color = self.WHITE
            text_color = self.BLACK
            hover_color = self.HOVER_COLOR_WHITE
        else:
            default_color = self.BLACK
            text_color = self.WHITE
            hover_color = self.HOVER_COLOR_BLACK

        if surrender_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_surrender.move(3, 3), border_radius=12)
        surrender_color = hover_color if surrender_hover else default_color
        pygame.draw.rect(self.canvas, surrender_color, self.button_rect_surrender, border_radius=12)
        surrender_text = self.small_font.render("Poddaj się", True, text_color)
        self.canvas.blit(surrender_text, (
            self.button_rect_surrender.x + (self.button_rect_surrender.width - surrender_text.get_width()) // 2,
            self.button_rect_surrender.y + (self.button_rect_surrender.height - surrender_text.get_height()) // 2
        ))

        if draw_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_draw.move(3, 3), border_radius=12)
        draw_color = hover_color if draw_hover else default_color
        pygame.draw.rect(self.canvas, draw_color, self.button_rect_draw, border_radius=12)
        draw_text = self.small_font.render("Zaproponuj remis", True, text_color)
        self.canvas.blit(draw_text, (
            self.button_rect_draw.x + (self.button_rect_draw.width - draw_text.get_width()) // 2,
            self.button_rect_draw.y + (self.button_rect_draw.height - draw_text.get_height()) // 2
        ))

        if undo_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_undo.move(3, 3), border_radius=12)
        undo_color = hover_color if undo_hover else default_color
        pygame.draw.rect(self.canvas, undo_color, self.button_rect_undo, border_radius=12)
        undo_text = self.small_font.render("Cofnij ruch", True, text_color)
        self.canvas.blit(undo_text, (
            self.button_rect_undo.x + (self.button_rect_undo.width - undo_text.get_width()) // 2,
            self.button_rect_undo.y + (self.button_rect_undo.height - undo_text.get_height()) // 2
        ))

        if nerd_hover:
            pygame.draw.rect(self.canvas, self.SHADOW_COLOR, self.button_rect_nerd.move(3, 3), border_radius=12)
        nerd_color = hover_color if nerd_hover else default_color
        pygame.draw.rect(self.canvas, nerd_color, self.button_rect_nerd, border_radius=12)
        nerd_text = self.small_font.render("Tryb Nerda", True, text_color)
        self.canvas.blit(nerd_text, (
            self.button_rect_nerd.x + (self.button_rect_nerd.width - nerd_text.get_width()) // 2,
            self.button_rect_nerd.y + (self.button_rect_nerd.height - nerd_text.get_height()) // 2
        ))

        if not self.game_started:
            overlay = pygame.Surface(self.total_size)
            overlay.set_alpha(200)
            overlay.fill((0, 0, 0))
            self.canvas.blit(overlay, (0, 0))

            waiting_text = self.font.render("Oczekiwanie na gracza...", True, (255, 255, 255))
            text_rect = waiting_text.get_rect(center=(self.total_size[0] // 2, self.total_size[1] // 2 - 50))
            self.canvas.blit(waiting_text, text_rect)

            game_code_text = self.small_font.render(f"Kod gry: {self.room_id}", True, (255, 255, 255))
            game_code_rect = game_code_text.get_rect(center=(self.total_size[0] // 2, self.total_size[1] // 2 - 20))
            self.canvas.blit(game_code_text, game_code_rect)

            self.button_rect_cancel = pygame.Rect(self.total_size[0] // 2 - 100, self.total_size[1] // 2 + 20, 200, 40)
            cancel_hover = self.button_rect_cancel.collidepoint(mouse_pos)
            if cancel_hover:
                pygame.draw.rect(self.canvas, (100, 100, 100), self.button_rect_cancel.move(3, 3), border_radius=12)
            cancel_color = (70, 70, 70) if cancel_hover else (50, 50, 50)
            pygame.draw.rect(self.canvas, cancel_color, self.button_rect_cancel, border_radius=12)
            cancel_text = self.small_font.render("Anuluj", True, (255, 255, 255))
            self.canvas.blit(cancel_text, (
                self.button_rect_cancel.x + (self.button_rect_cancel.width - cancel_text.get_width()) // 2,
                self.button_rect_cancel.y + (self.button_rect_cancel.height - cancel_text.get_height()) // 2
            ))


        if self.draw_request_pending and self.draw_requester != "player":
            draw_prompt_rect = pygame.Rect(self.total_size[0] // 2 - 175, self.total_size[1] // 2 - 60, 350, 120)
            pygame.draw.rect(self.canvas, (255, 255, 255), draw_prompt_rect, border_radius=10)
            pygame.draw.rect(self.canvas, (0, 0, 0), draw_prompt_rect, 2, border_radius=10)

            prompt_text = self.small_font.render("Przeciwnik proponuje remis", True, (0, 0, 0))
            self.canvas.blit(prompt_text, (draw_prompt_rect.x + (draw_prompt_rect.width - prompt_text.get_width()) // 2,
                                           draw_prompt_rect.y + 20))

            mouse_pos = pygame.mouse.get_pos()
            # Poszerz przyciski z 130 na 140
            self.button_rect_accept = pygame.Rect(draw_prompt_rect.x + 40, draw_prompt_rect.y + 60, 140, 40)
            self.button_rect_reject = pygame.Rect(draw_prompt_rect.x + 190, draw_prompt_rect.y + 60, 140, 40)

            accept_hover = self.button_rect_accept.collidepoint(mouse_pos)
            reject_hover = self.button_rect_reject.collidepoint(mouse_pos)

            accept_color = (200, 255, 200) if accept_hover else (150, 255, 150)
            reject_color = (255, 200, 200) if reject_hover else (255, 150, 150)

            pygame.draw.rect(self.canvas, accept_color, self.button_rect_accept, border_radius=8)
            pygame.draw.rect(self.canvas, reject_color, self.button_rect_reject, border_radius=8)

            accept_text = self.small_font.render("Zaakceptuj", True, (0, 0, 0))
            reject_text = self.small_font.render("Odrzuć", True, (0, 0, 0))

            self.canvas.blit(accept_text, (
                self.button_rect_accept.x + (self.button_rect_accept.width - accept_text.get_width()) // 2,
                self.button_rect_accept.y + (self.button_rect_accept.height - accept_text.get_height()) // 2
            ))
            self.canvas.blit(reject_text, (
                self.button_rect_reject.x + (self.button_rect_reject.width - reject_text.get_width()) // 2,
                self.button_rect_reject.y + (self.button_rect_reject.height - reject_text.get_height()) // 2
            ))

        if self.undo_request_pending and self.undo_requester != "player":
            # Zachowaj wymiary z poprzedniej wersji: 350x140, y na -70
            prompt_rect = pygame.Rect(self.total_size[0] // 2 - 175, self.total_size[1] // 2 - 70, 350, 140)
            pygame.draw.rect(self.canvas, (255, 255, 255), prompt_rect, border_radius=10)
            pygame.draw.rect(self.canvas, (0, 0, 0), prompt_rect, 2, border_radius=10)

            # Przesuń tekst bardziej do góry
            prompt_text1 = self.small_font.render("Przeciwnik chce cofnąć", True, (0, 0, 0))
            prompt_text2 = self.small_font.render("swój ostatni ruch", True, (0, 0, 0))
            self.canvas.blit(prompt_text1, (prompt_rect.x + (prompt_rect.width - prompt_text1.get_width()) // 2,
                                            prompt_rect.y + 10))  # Z 30 na 10 - bardziej do góry
            self.canvas.blit(prompt_text2, (prompt_rect.x + (prompt_rect.width - prompt_text2.get_width()) // 2,
                                            prompt_rect.y + 40))  # Z 60 na 40 - bardziej do góry

            mouse_pos = pygame.mouse.get_pos()
            # Przesuń przyciski bardziej w dół
            self.button_rect_accept = pygame.Rect(prompt_rect.x + 40, prompt_rect.y + 90, 140, 40)  # Z 80 na 90 - lekko w dół
            self.button_rect_reject = pygame.Rect(prompt_rect.x + 190, prompt_rect.y + 90, 140, 40)  # Z 80 na 90 - lekko w dół

            accept_hover = self.button_rect_accept.collidepoint(mouse_pos)
            reject_hover = self.button_rect_reject.collidepoint(mouse_pos)

            accept_color = (200, 255, 200) if accept_hover else (150, 255, 150)
            reject_color = (255, 200, 200) if reject_hover else (255, 150, 150)

            pygame.draw.rect(self.canvas, accept_color, self.button_rect_accept, border_radius=8)
            pygame.draw.rect(self.canvas, reject_color, self.button_rect_reject, border_radius=8)

            accept_text = self.small_font.render("Zaakceptuj", True, (0, 0, 0))
            reject_text = self.small_font.render("Odrzuć", True, (0, 0, 0))

            self.canvas.blit(accept_text, (
                self.button_rect_accept.x + (self.button_rect_accept.width - accept_text.get_width()) // 2,
                self.button_rect_accept.y + (self.button_rect_accept.height - accept_text.get_height()) // 2
            ))
            self.canvas.blit(reject_text, (
                self.button_rect_reject.x + (self.button_rect_reject.width - reject_text.get_width()) // 2,
                self.button_rect_reject.y + (self.button_rect_reject.height - reject_text.get_height()) // 2
            ))

        if self.current_notification:
            notification_width = self.total_size[0] - 20  # Almost full width with 10px margin on each side
            notification_x = 10  # 10px margin from left
            notification_y = self.total_size[1] - self.notification_height - 10 + int(self.notification_y_offset)

            # Draw notification background with shadow
            shadow_rect = pygame.Rect(notification_x + 3, notification_y + 3, notification_width, self.notification_height)
            pygame.draw.rect(self.canvas, (50, 50, 50), shadow_rect, border_radius=10)
            notification_rect = pygame.Rect(notification_x, notification_y, notification_width, self.notification_height)
            pygame.draw.rect(self.canvas, (200, 200, 200), notification_rect, border_radius=10)
            pygame.draw.rect(self.canvas, (0, 0, 0), notification_rect, 2, border_radius=10)

            # Render notification text with bold font
            notification_text = self.bold_small_font.render(self.current_notification, True, (0, 0, 0))
            text_rect = notification_text.get_rect(center=notification_rect.center)
            self.canvas.blit(notification_text, text_rect)

        if self.game_over:
            self.render_game_over()
        pygame.display.update()


    def cleanup(self):
        self.running = False
        if self.ws:
            self.ws.close()
        if self.ws_thread:
            self.ws_thread.join(timeout=1)

    def run(self):
        last_ping_time = 0
        while self.running:
            self.process_messages()
            self.process_events()

            if self.return_to_menu:
                self.cleanup()
                return

            current_time = time.time()
            if self.connected and current_time - last_ping_time >= self.ping_interval:
                self.send_ping()
                last_ping_time = current_time

            if self.game_started and not self.game_over and not self.animating and not self.promotion_pending and self.last_time_update is not None:
                current_time = pygame.time.get_ticks()
                elapsed = (current_time - self.last_time_update) / 1000
                if self.turn == "white" and self.white_time > 0:
                    self.white_time -= elapsed
                elif self.turn == "black" and self.black_time > 0:
                    self.black_time -= elapsed
                self.last_time_update = current_time

            self.update()
            self.render()
            self.clock.tick(60)
