import os, requests, sys
import game
import pygame
import time
import style
from board import Board

# Funkcje z login.py (bez zmian)
def save_refresh_token(token):
    lines = []
    try:
        if os.path.exists(TOKEN_FILE):
            with open(TOKEN_FILE, "r", encoding="utf-8") as file:
                lines = file.readlines()
    except Exception as e:
        print(f"Błąd odczytu pliku: {e}")
        lines = []

    while len(lines) < 2:
        lines.append("\n")

    lines[1] = str(token) + "\n"

    try:
        with open(TOKEN_FILE, "w", encoding="utf-8") as file:
            file.writelines(lines)
    except Exception as e:
        print(f"Błąd zapisu do pliku: {e}")

def load_refresh_token():
    if os.path.exists(TOKEN_FILE):
        with open(TOKEN_FILE, "r", encoding="utf-8") as file:
            lines = file.readlines()
        if len(lines) >= 2:
            return lines[1].strip()
    return None

def delete_refresh_token():
    if os.path.exists(TOKEN_FILE):
        with open(TOKEN_FILE, "r") as file:
            lines = file.readlines()

        if len(lines) > 1:
            lines[1] = "\n"

        with open(TOKEN_FILE, "w") as file:
            file.writelines(lines)

def api_request(endpoint, method="GET", data=None, headers=None, access_token=None):
    url = f"{API_BASE}/{endpoint}"
    print(url)
    default_headers = {"Authorization": f"Bearer {access_token}"} if access_token else {}
    final_headers = {**default_headers, **(headers or {})}
    try:
        response = requests.request(method, url, json=data, headers=final_headers)
        return response
    except requests.exceptions.RequestException:
        return None

def refresh_access_token():
    refresh_token = load_refresh_token()
    if not refresh_token:
        return None
    response = api_request("refresh", "POST", {"refresh_token": refresh_token})
    if response is None or response.status_code != 200:
        delete_refresh_token()
        return None
    tokens = response.json()
    save_refresh_token(tokens["refresh_token"])
    return tokens["access_token"]

def get_user_info(access_token):
    response = api_request("user", "GET", access_token=access_token)
    if response is None:
        return "Błąd połączenia z serwerem"
    if response.status_code == 200:
        return response.json()
    elif response.status_code == 401:
        return "Nieautoryzowany"
    return "Błąd serwera"

def create_room_request(access_token, room_name, is_public):
    data = {"name": room_name, "is_public": is_public}
    response = api_request("rooms", "POST", data, access_token=access_token)
    if response is None:
        return "Błąd połączenia z serwerem"
    if response.status_code == 200:
        join_room(response.json().get("id"), access_token)
        return "OK"
    elif response.status_code == 401:
        return "Nieautoryzowany - odśwież token"
    return response.json().get("message", "Błąd")

def get_rooms(access_token):
    response = api_request("rooms", "GET", access_token=access_token)
    if response is None or response.status_code != 200:
        return []
    return response.json() if response.json() is not None else []

def join_room(room_id, access_token):
    global WIDTH, HEIGHT, square_size, board
    print(f"Joining room with ID: {room_id}")
    game.ChessGame(room_id, access_token, api_request=api_request).run()

TOKEN_FILE = ""
HOME_DIR = os.path.expanduser("~")
if len(sys.argv) > 1:
    TOKEN_FILE = sys.argv[1]
else:
    TOKEN_FILE = os.path.join(HOME_DIR, ".azbest_chess")
API_BASE = "https://azbest.org.pl"

access_token = refresh_access_token()
if access_token is None:
    print("Unauthorized")
    exit()

user_info = get_user_info(access_token)
if user_info == "Nieautoryzowany":
    access_token = refresh_access_token()
    if access_token:
        user_info = get_user_info(access_token)
    else:
        user_info = None

if user_info is None:
    print("Unauthorized")
    exit()

pygame.init()

# --- Ustawienia ekranu ---
WIDTH, HEIGHT = 940, 640  # Stały rozmiar okna 940x640
square_size = 80          # Domyślny rozmiar pola z board.py
rows = HEIGHT // square_size  # 8 pól w pionie (640 / 80 = 8)
cols = WIDTH // square_size   # 11 pól w poziomie (940 / 80 = 11, reszta 60 pikseli)
screen = pygame.display.set_mode((WIDTH, HEIGHT), pygame.RESIZABLE)
pygame.display.set_caption("SZACHY ONLINE - Motorola Science Cup 2025")

# --- Kolory ---
BG_COLOR = (50, 50, 50)  # Fallback
WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
HOVER_COLOR = (100, 100, 100)
BUTTON_COLOR = (200, 200, 200)
SELECTED_COLOR = (0, 150, 0)
ERROR_COLOR = (255, 50, 50)
BOARD_LIGHT = (240, 217, 181)  # Jasny kolor szachownicy
BOARD_DARK = (181, 136, 99)   # Ciemny kolor szachownicy

# --- Czcionki ---
pygame.font.init()
font = pygame.font.Font(style.MONTSERRAT_BOLD, 40)
small_font = pygame.font.Font(style.MONTSERRAT_REGULAR, 25)
error_font = pygame.font.Font(style.MONTSERRAT_BOLD, 25)  # Pogrubiona czcionka dla błędów

# --- Inicjalizacja szachownicy jako tła ---
board = Board(square_size=square_size, rows=rows, cols=cols, sidebar_width=0)
board.total_size = (WIDTH, HEIGHT)  # Dopasuj do rozmiaru okna

# --- Stałe ---
MAX_ROOM_NAME_LENGTH = 10
ROOM_CODE_LENGTH = 6

# --- Menu ekranów ---
MENU = "main"
BUTTONS = []

# --- Zmienna do śledzenia ostatniego odświeżenia listy pokoi ---
last_room_refresh = 0
rooms_cache = []

def draw_button(text, y_offset):
    """Tworzy przycisk i zwraca jego prostokąt."""
    button_width, button_height = 300, 70
    x = (WIDTH - button_width) // 2
    y = HEIGHT // 3 + y_offset
    rect = pygame.Rect(x, y, button_width, button_height)
    BUTTONS.append((text, rect))
    return rect

def render_text(text, font, color, center):
    """Renderuje tekst na środku podanej pozycji."""
    text_surface = font.render(text, True, color)
    text_rect = text_surface.get_rect(center=center)
    screen.blit(text_surface, text_rect)

def draw_background():
    """Rysuje tło z szachownicą i koloruje margines jak kontynuację szachownicy."""
    # Wypełnij całe tło kolorem ciemnych pól (kontynuacja wzoru)
    screen.fill(BOARD_DARK)

    # Rysuj szachownicę w środku
    board_x = (WIDTH - cols * square_size) // 2  # Wyśrodkowanie w poziomie
    board_y = (HEIGHT - rows * square_size) // 2  # Wyśrodkowanie w pionie
    for row in range(rows):
        for col in range(cols):
            color = BOARD_LIGHT if (row + col) % 2 == 0 else BOARD_DARK
            rect = pygame.Rect(board_x + col * square_size, board_y + row * square_size,
                              square_size, square_size)
            pygame.draw.rect(screen, color, rect)

def draw_main_menu():
    """Rysuje ekran głównego menu z tłem szachownicy."""
    global BUTTONS
    BUTTONS = []  # Reset listy przycisków

    # Rysuj tło z szachownicą i marginesem
    draw_background()

    # Rysuj elementy menu (tytuł czarny)
    render_text("Szachy Online", font, BLACK, (WIDTH // 2, 100))

    create_room_btn = draw_button("Stwórz pokój", 0)
    join_game_btn = draw_button("Dołącz do gry", 90)

    mouse_pos = pygame.mouse.get_pos()

    for text, rect in BUTTONS:
        color = HOVER_COLOR if rect.collidepoint(mouse_pos) else BUTTON_COLOR
        pygame.draw.rect(screen, color, rect, border_radius=10)
        pygame.draw.rect(screen, BLACK, rect, 2, border_radius=10)
        render_text(text, small_font, BLACK, rect.center)

def draw_create_room(room_name, is_public, error_message=""):
    """Rysuje ekran formularza tworzenia pokoju z tłem szachownicy."""
    draw_background()  # Tło szachownicy z marginesem
    render_text("Stwórz pokój", font, BLACK, (WIDTH // 2, 100))  # Tytuł czarny

    render_text("Nazwa pokoju:", small_font, BLACK, (WIDTH // 2, 200))  # Etykieta czarna
    input_rect = pygame.Rect(WIDTH // 2 - 150, 230, 300, 40)
    pygame.draw.rect(screen, WHITE, input_rect, border_radius=5)
    render_text(room_name, small_font, BLACK, (input_rect.centerx, input_rect.centery))

    render_text("Czy publiczne:", small_font, BLACK, (WIDTH // 2, 300))  # Etykieta czarna
    checkbox_rect = pygame.Rect(WIDTH // 2 + 100, 285, 20, 20)
    pygame.draw.rect(screen, WHITE, checkbox_rect, 2)

    if is_public:
        pygame.draw.line(screen, WHITE, (checkbox_rect.left + 4, checkbox_rect.centery),
                        (checkbox_rect.right - 4, checkbox_rect.centery), 3)
        pygame.draw.line(screen, WHITE, (checkbox_rect.centerx, checkbox_rect.top + 4),
                        (checkbox_rect.centerx, checkbox_rect.bottom - 4), 3)

    back_button = pygame.Rect(WIDTH // 2 - 150, 400, 150, 60)
    pygame.draw.rect(screen, BUTTON_COLOR, back_button, border_radius=10)
    pygame.draw.rect(screen, BLACK, back_button, 2, border_radius=10)
    render_text("Powrót", small_font, BLACK, back_button.center)

    create_button = pygame.Rect(WIDTH // 2 + 10, 400, 150, 60)
    mouse_pos = pygame.mouse.get_pos()
    create_color = HOVER_COLOR if create_button.collidepoint(mouse_pos) else BUTTON_COLOR
    pygame.draw.rect(screen, create_color, create_button, border_radius=10)
    pygame.draw.rect(screen, BLACK, create_button, 2, border_radius=10)
    render_text("Stwórz", small_font, BLACK, create_button.center)

    if error_message:
        render_text(error_message, error_font, ERROR_COLOR, (WIDTH // 2, 480))

    return back_button, checkbox_rect, create_button

def draw_join_game(room_code="", scroll_offset=0, error_message=""):
    """Rysuje ekran listy dostępnych pokoi z tłem szachownicy."""
    global last_room_refresh, rooms_cache
    draw_background()  # Tło szachownicy z marginesem
    render_text("Publiczne pokoje", font, BLACK, (WIDTH // 2, 50))  # Tytuł czarny

    # Odświeżanie listy pokoi co 2 sekundy
    current_time = time.time()
    if current_time - last_room_refresh >= 2:
        rooms_cache = get_rooms(access_token)
        last_room_refresh = current_time

    room_buttons = []

    # Obszar przewijania
    visible_height = 300
    room_height = 60
    total_height = len(rooms_cache) * room_height
    start_y = 100 - scroll_offset

    for i, room in enumerate(rooms_cache):
        y = start_y + i * room_height
        if y + room_height < 100 or y > 100 + visible_height:
            continue
        rect = pygame.Rect(WIDTH // 2 - 150, y, 300, 50)
        color = HOVER_COLOR if rect.collidepoint(pygame.mouse.get_pos()) else WHITE
        pygame.draw.rect(screen, color, rect, border_radius=5)
        pygame.draw.rect(screen, BLACK, rect, 2, border_radius=5)
        room_text = f"{room['name']}"
        render_text(room_text, small_font, BLACK, rect.center)
        room_buttons.append((room['id'], rect))

    # Pole do wprowadzania kodu pokoju prywatnego
    render_text("Kod pokoju:", small_font, BLACK, (WIDTH // 2, 420))  # Etykieta czarna
    code_input_rect = pygame.Rect(WIDTH // 2 - 150, 450, 200, 40)
    pygame.draw.rect(screen, WHITE, code_input_rect, border_radius=5)
    render_text(room_code, small_font, BLACK, (code_input_rect.centerx, code_input_rect.centery))

    # Przycisk dołącz dla kodu
    join_button = pygame.Rect(WIDTH // 2 + 60, 450, 90, 40)
    mouse_pos = pygame.mouse.get_pos()
    join_color = HOVER_COLOR if join_button.collidepoint(mouse_pos) else BUTTON_COLOR
    pygame.draw.rect(screen, join_color, join_button, border_radius=10)
    pygame.draw.rect(screen, BLACK, join_button, 2, border_radius=10)
    render_text("Dołącz", small_font, BLACK, join_button.center)

    # Przycisk powrotu
    back_button = pygame.Rect(WIDTH // 2 - 150, HEIGHT - 80, 300, 60)
    back_color = HOVER_COLOR if back_button.collidepoint(mouse_pos) else BUTTON_COLOR
    pygame.draw.rect(screen, back_color, back_button, border_radius=10)
    pygame.draw.rect(screen, BLACK, back_button, 2, border_radius=10)
    render_text("Powrót", small_font, BLACK, back_button.center)

    # Etykieta błędu (pogrubiona)
    if error_message:
        error_y = HEIGHT - 10
        render_text(error_message, error_font, ERROR_COLOR, (WIDTH // 2, error_y))

    return back_button, room_buttons, join_button, code_input_rect, min(0, max(total_height - visible_height, 0))

# --- Pętla gry ---
running = True
room_name = ""
is_public = True
error_message = ""
room_code = ""
scroll_offset = 0

while running:
    mouse_pos = pygame.mouse.get_pos()

    if MENU == "main":
        draw_main_menu()
    elif MENU == "create_room":
        back_button, checkbox_rect, create_button = draw_create_room(room_name, is_public, error_message)
    elif MENU == "join_game":
        back_button, room_buttons, join_button, code_input_rect, max_scroll = draw_join_game(room_code, scroll_offset, error_message)

    pygame.display.flip()

    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            pygame.quit()
            sys.exit()
        elif event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
            if MENU == "main":
                for text, rect in BUTTONS:
                    if rect.collidepoint(mouse_pos):
                        if text == "Stwórz pokój":
                            MENU = "create_room"
                            room_name = ""
                            is_public = True
                            error_message = ""
                        elif text == "Dołącz do gry":
                            MENU = "join_game"
                            room_code = ""
                            scroll_offset = 0
                            error_message = ""
            elif MENU == "create_room":
                if back_button.collidepoint(mouse_pos):
                    MENU = "main"
                    error_message = ""
                elif checkbox_rect.collidepoint(mouse_pos):
                    is_public = not is_public
                elif create_button.collidepoint(mouse_pos):
                    error_message = create_room_request(access_token, room_name, is_public)
            elif MENU == "join_game":
                if back_button.collidepoint(mouse_pos):
                    MENU = "main"
                    error_message = ""
                for room_id, rect in room_buttons:
                    if rect.collidepoint(mouse_pos):
                        join_room(room_id, access_token)
                        error_message = f"Dołączono do pokoju: {room_id}"
                if join_button.collidepoint(mouse_pos):
                    if len(room_code) == ROOM_CODE_LENGTH and room_code.isdigit():
                        join_room(room_code, access_token)
                        error_message = f"Dołączono do pokoju: {room_code}"
                    else:
                        error_message = "Kod musi być 6-cyfrowy"
        elif event.type == pygame.MOUSEWHEEL and MENU == "join_game":
            scroll_offset = max(0, min(scroll_offset - event.y * 20, max_scroll))
        elif event.type == pygame.KEYDOWN:
            if MENU == "create_room":
                if event.key == pygame.K_BACKSPACE:
                    room_name = room_name[:-1]
                elif event.key == pygame.K_RETURN:
                    error_message = create_room_request(access_token, room_name, is_public)
                elif len(room_name) < MAX_ROOM_NAME_LENGTH:
                    if event.unicode.isalnum() or event.unicode == " ":
                        room_name += event.unicode
            elif MENU == "join_game" and code_input_rect.collidepoint(mouse_pos):
                if event.key == pygame.K_BACKSPACE:
                    room_code = room_code[:-1]
                elif event.key == pygame.K_RETURN:
                    if len(room_code) == ROOM_CODE_LENGTH and room_code.isdigit():
                        join_room(room_code, access_token)
                        error_message = f"Dołączono do pokoju: {room_code}"
                    else:
                        error_message = "Kod musi być 6-cyfrowy"
                elif len(room_code) < ROOM_CODE_LENGTH and event.unicode.isdigit():
                    room_code += event.unicode

pygame.quit()
