import os, pygame


def asset_path(filename):
    current_dir = os.path.dirname(os.path.abspath(__file__))
    assets_dir = os.path.join(current_dir, "..", "assets")
    assets_dir = os.path.normpath(assets_dir)
    return os.path.join(assets_dir, filename)


def pos_to_alg(pos):
    col, row = pos
    letter = chr(ord('a') + col)
    rank = str(8 - row)
    return letter + rank

def pos_from_alg(alg):
    letter, rank = alg[0], alg[1]
    col = ord(letter) - ord('a')
    row = 8 - int(rank)
    return (col, row)

def load_image(filename, square_size, scale_factor=0.9):
    path = asset_path(filename)
    if not os.path.exists(path):
        print(f"Error: Image file not found: {path}")
        return None
    try:
        img = pygame.image.load(path)
        new_size = (int(square_size * scale_factor), int(square_size * scale_factor))
        return pygame.transform.scale(img, new_size)
    except pygame.error as e:
        print(f"Could not load image {path}: {e}")
        return None

def is_inside(col, row):
    return 0 <= col < 8 and 0 <= row < 8
