import pygame
import sys
import os
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))  # Dodajemy katalog client do ścieżki
import settings  # Importujemy settings z client
import style

class Board:
    def __init__(self, square_size=80, rows=8, cols=8, sidebar_width=300):
        self.rows = rows
        self.cols = cols
        self.square_size = square_size
        self.board_width = self.cols * self.square_size
        self.board_height = self.rows * self.square_size
        
        # Pobieramy kolory z settings.CHESS_SETS na podstawie CURRENT_CHESS_SET
        self.light_color = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][0]  # Jasny kolor
        self.dark_color = settings.CHESS_SETS[settings.CURRENT_CHESS_SET][1]   # Ciemny kolor
        
        self.sidebar_width = sidebar_width
        self.total_size = (self.board_width + self.sidebar_width, self.board_height)
        # Czcionka do podpisów
        self.font = pygame.font.Font(style.MONTSERRAT_BOLD, int(self.square_size / 3))

    def draw(self, canvas):
        # Draw the chessboard squares
        for row in range(self.rows):
            for col in range(self.cols):
                color = self.light_color if (row + col) % 2 == 0 else self.dark_color
                rect = pygame.Rect(col * self.square_size, row * self.square_size,
                                   self.square_size, self.square_size)
                pygame.draw.rect(canvas, color, rect)
        # Draw the board border
        border_rect = pygame.Rect(0, 0, self.board_width, self.board_height)
        pygame.draw.rect(canvas, (0, 0, 0), border_rect, 2)

        # PODPISYWANIE POL
        CHESS_LIGHT = self.light_color  # Jasny kolor z ustawień
        CHESS_DARK = self.dark_color    # Ciemny kolor z ustawień

        # Cyfry (1-8) po lewej stronie w lewym górnym rogu pól
        for row in range(self.rows):
            number = str(8 - row)  # Od 8 do 1 (od góry do dołu)
            text_color = CHESS_DARK if row % 2 == 0 else CHESS_LIGHT
            left_text = self.font.render(number, True, text_color)
            left_rect = left_text.get_rect(topleft=(5, row * self.square_size + 5))
            canvas.blit(left_text, left_rect)

        # Litery (a-h) na dole w prawym dolnym rogu pól
        for col in range(self.cols):
            letter = chr(97 + col)  # 'a' = 97 w ASCII
            text_color = CHESS_LIGHT if col % 2 == 0 else CHESS_DARK
            bottom_text = self.font.render(letter, True, text_color)
            bottom_rect = bottom_text.get_rect(bottomright=(col * self.square_size + self.square_size - 5,
                                                            self.board_height - 5))
            canvas.blit(bottom_text, bottom_rect)  # Poprawiono na bottom_text

if __name__ == "__main__":
    pygame.init()
    screen = pygame.display.set_mode((800, 600))
    board = Board()
    running = True
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
        screen.fill((255, 255, 255))  # Białe tło
        board.draw(screen)
        pygame.display.flip()
    pygame.quit()