import pygame
import sys
import game
import os

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
import settings
import mainmenu
import style

pygame.init()

# Konfiguracja ekranu
WIDTH, HEIGHT = settings.WIDTH, settings.HEIGHT
screen = pygame.display.set_mode((WIDTH, HEIGHT), pygame.RESIZABLE)
pygame.display.set_caption("WYBÓR TRYBU GRY - Motorola Science Cup 2025")

# Czcionki
bfont = pygame.font.Font(style.MONTSERRAT_BOLD, 40)
buttonfont = pygame.font.Font(style.MONTSERRAT_SEMI_BOLD, 37)
font = pygame.font.Font(style.MONTSERRAT_BOLD, 37)

# Kolory
WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
GRAY = (200, 200, 200)
DARK_GRAY = (80, 80, 80)
HOVER_COLOR = settings.HOVER_COLOR

# Tryby gry
MODES = {
    "Klasyczny": "classic",
    "Błyskawiczny": "blitz"
}

def draw_button(screen, text, selFont, rect, color, hover_color):
    """Rysuje przycisk z efektem hover"""
    mouse = pygame.mouse.get_pos()
    hover = rect.collidepoint(mouse)
    
    # Przycisk
    button_color = hover_color if hover else color
    pygame.draw.rect(screen, button_color, rect, border_radius=10)
    pygame.draw.rect(screen, WHITE, rect, 2, border_radius=10)
    
    # Tekst
    text_surf = selFont.render(text, True, WHITE)
    text_rect = text_surf.get_rect(center=rect.center)
    screen.blit(text_surf, text_rect)

def draw_mode_selection():
    """Rysuje ekran wyboru trybu z dynamicznym dostosowaniem elementów."""
    settings.load_settings()
    settings.draw_chessboard(screen, screen.get_width(), screen.get_height())

    current_width, current_height = screen.get_size()

    # Tło dla nagłówka z szarym kolorem
    title_text = bfont.render("Wybierz Tryb Rozgrywki", True, WHITE)
    title_width = title_text.get_width() + 40
    title_height = title_text.get_height() + 20
    title_x = current_width // 2 - title_width // 2
    title_y = 40
    title_rect = pygame.Rect(title_x, title_y, title_width, title_height)
    
    pygame.draw.rect(screen, DARK_GRAY, title_rect, border_radius=10)  # Szare tło
    pygame.draw.rect(screen, WHITE, title_rect, 2, border_radius=10)  # Obramowanie
    screen.blit(title_text, (current_width // 2 - title_text.get_width() // 2, 50))

    # Przyciski trybów gry
    mode_rects = []
    y_offset = current_height // 3
    button_width = int(current_width * 0.35)
    button_height = int(current_height * 0.15)

    for mode_name in MODES.keys():
        rect = pygame.Rect(current_width // 2 - button_width // 2, y_offset, button_width, button_height)
        draw_button(screen, mode_name, buttonfont, rect, DARK_GRAY, HOVER_COLOR)
        mode_rects.append((rect, mode_name))
        y_offset += button_height + 20

    # Przycisk powrotu
    back_button_rect = pygame.Rect(current_width // 2 - button_width // 2, y_offset + 20, button_width, button_height // 2)
    draw_button(screen, "Wyjdź", font, back_button_rect, DARK_GRAY, HOVER_COLOR)

    return mode_rects, back_button_rect

def mode_selection_screen():
    """Obsługuje ekran wyboru trybu gry."""
    mouse_released = False

    while True:
        mode_rects, back_button_rect = draw_mode_selection()
        pygame.display.update()

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()

            if event.type == pygame.MOUSEBUTTONDOWN and event.button == 1:
                mouse_released = False

            if event.type == pygame.MOUSEBUTTONUP and event.button == 1 and not mouse_released:
                mouse_pos = pygame.mouse.get_pos()
                mouse_released = True

                if back_button_rect.collidepoint(mouse_pos):
                    pygame.quit()
                    sys.exit()

                for rect, mode_name in mode_rects:
                    if rect.collidepoint(mouse_pos):
                        start_game(MODES[mode_name])

def start_game(game_mode):
    game_instance = game.Game(game_mode=game_mode)
    game_instance.run()

if __name__ == "__main__":
    mode_selection_screen()