package main

import (
	"crypto/rand"
	"database/sql"
	"encoding/base64"
	"encoding/json"
	"net/http"
	"time"

	"github.com/golang-jwt/jwt/v5"
)

var jwtSecret = []byte("")

type Claims struct {
	UserID int `json:"user_id"`
	jwt.RegisteredClaims
}

func generateRefreshToken() (string, error) {
	b := make([]byte, 32) // 256-bit token
	_, err := rand.Read(b)
	if err != nil {
		return "", err
	}
	return base64.RawURLEncoding.EncodeToString(b), nil
}

func getUserIDFromContext(r *http.Request) int {
	userID, ok := r.Context().Value(userIDKey).(int)
	if !ok {
		return 0
	}
	return userID
}

func getUserIDFromToken(tokenString string) int {
	token, err := jwt.ParseWithClaims(tokenString, &Claims{}, func(token *jwt.Token) (interface{}, error) {
		return jwtSecret, nil
	})

	if err != nil {
		return 0
	}

	claims, ok := token.Claims.(*Claims)
	if !ok || !token.Valid {
		return 0
	}

	return claims.UserID
}

func createAccessToken(userID int) (string, error) {
	claims := Claims{
		UserID: userID,
		RegisteredClaims: jwt.RegisteredClaims{
			ExpiresAt: jwt.NewNumericDate(time.Now().Add(60 * time.Minute)),
		},
	}
	token := jwt.NewWithClaims(jwt.SigningMethodHS256, claims)
	return token.SignedString(jwtSecret)
}

func createRefreshToken(db *sql.DB, userID int, ip string, parentID int) (string, error) {
	token, err := generateRefreshToken()
	if err != nil {
		return "", err
	}

	_, err = db.Exec("INSERT INTO refresh_tokens (user_id, token, expires_at,ip_address,parent_token_id) VALUES (?, ?, ?,?,?)",
		userID, token, time.Now().Add(30*24*time.Hour), ip, parentID)
	if err != nil {
		return "", err
	}

	return token, nil
}

func refreshTokenHandler(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	r.Body = http.MaxBytesReader(w, r.Body, MAX_REQ_SIZE)

	var data struct {
		Token string `json:"refresh_token"`
	}

	if err := json.NewDecoder(r.Body).Decode(&data); err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusBadRequest)
		return
	}

	var userID int
	var parentTokenID int
	err := db.QueryRow("SELECT id,user_id FROM refresh_tokens WHERE used_at IS NULL AND deactivated = 0 AND expires_at > CURRENT_TIMESTAMP AND token = ?", data.Token).Scan(&parentTokenID, &userID)
	if err != nil {
		sendErrorResponse(w, "Niepoprawny token", http.StatusUnauthorized)
		return
	}

	_, err = db.Exec("UPDATE refresh_tokens SET used_at = CURRENT_TIMESTAMP WHERE token = ?", data.Token)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	accessToken, err := createAccessToken(userID)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	refreshToken, err := createRefreshToken(db, userID, GetIP(r), parentTokenID)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}

	response := LoginData{
		Id:           userID,
		RefreshToken: refreshToken,
		AccessToken:  accessToken,
	}

	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusOK)
	json.NewEncoder(w).Encode(response)
}
