package main

import (
	"database/sql"
	"encoding/json"
	"net/http"
	"strconv"
	"time"
)

func rankingHandler(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	r.Body = http.MaxBytesReader(w, r.Body, MAX_REQ_SIZE)

	limitVal := r.PathValue("limit")
	limit, err := strconv.Atoi(limitVal)
	if err != nil || limitVal == "" || limit < 1 {
		limit = 10
	}
	rows, err := db.Query("SELECT username, display_name,created_at, elo FROM users ORDER BY elo DESC LIMIT ?", limit)
	if err != nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}
	defer rows.Close()

	var users []SimpleUserProfile
	var dname string
	var createdAtFull time.Time
	for rows.Next() {
		var u SimpleUserProfile
		if err := rows.Scan(&u.Nick, &dname, &createdAtFull, &u.Elo); err != nil {
			sendErrorResponse(w, "Błąd przetwarzania danych", http.StatusInternalServerError)
			return
		}
		if dname != "" {
			u.Nick = dname
		}
		u.CreatedAt = createdAtFull.Format("2006-01-02")
		users = append(users, u)
	}

	if err := rows.Err(); err != nil {
		sendErrorResponse(w, "Błąd odczytu wyników", http.StatusInternalServerError)
		return
	}

	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusOK)
	json.NewEncoder(w).Encode(users)

}
