package main

import (
	"database/sql"
	"encoding/json"
	"net/http"
)

func roomsHandler(db *sql.DB, w http.ResponseWriter, r *http.Request, canBeFull bool) {
	r.Body = http.MaxBytesReader(w, r.Body, MAX_REQ_SIZE)

	// targetID := strings.Trim(r.PathValue("id"), " ")
	w.Header().Set("Content-Type", "application/json")

	roomsMu.RLock()
	defer roomsMu.RUnlock()

	var roomsToReturn []RoomSerializable
	for _, room := range rooms {
		if room.IsPublic && (canBeFull || (!canBeFull && !room.isFull())) {
			roomsToReturn = append(roomsToReturn, *room.toSerializable(db))
		}
	}
	w.WriteHeader(http.StatusOK)
	json.NewEncoder(w).Encode(roomsToReturn)
}

func createRoomHandler(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	r.Body = http.MaxBytesReader(w, r.Body, MAX_REQ_SIZE)
	userID := getUserIDFromContext(r)
	_, isUserAlreadyInRoom := rooms.isPlayerInRoom(userID)
	if isUserAlreadyInRoom {
		sendErrorResponse(w, "Już jesteś połączony z innym pokojem", http.StatusBadRequest)
		return
	}

	var req struct {
		Name     *string `json:"name"`
		IsPublic *bool   `json:"is_public"`
	}
	if err := json.NewDecoder(r.Body).Decode(&req); err != nil {
		sendErrorResponse(w, "Niepoprawne zapytanie", http.StatusBadRequest)
		return
	}

	name := ""
	if req.Name != nil {
		name = *req.Name
	}

	isPublic := true
	if req.IsPublic != nil {
		isPublic = *req.IsPublic
	}

	room := newRoom(db, name, isPublic, userID)
	if room == nil {
		sendErrorResponse(w, "Coś poszło nie tak", http.StatusInternalServerError)
		return
	}
	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusOK)
	json.NewEncoder(w).Encode(*(room.toSerializable(db)))
}
