package main

import (
	"database/sql"
	"encoding/json"
	"net/http"
	"strconv"
	"time"
)

type UserProfile struct {
	Username    string `json:"username"`
	DisplayName string `json:"display_name"`
	CreatedAt   string `json:"created_at"`
	Elo         string `json:"elo"`
}

func (pf *UserProfile) Simplify() SimpleUserProfile {
	nick := pf.DisplayName
	if nick == "" {
		nick = pf.Username
	}
	return SimpleUserProfile{
		CreatedAt: pf.CreatedAt,
		Nick:      nick,
		Elo:       pf.Elo,
	}
}

type SimpleUserProfile struct {
	Nick      string `json:"nick"`
	CreatedAt string `json:"created_at"`
	Elo       string `json:"elo"`
}

func getUserProfileByID(db *sql.DB, userID int) (UserProfile, error) {
	var createdAtFull time.Time

	var userResponse UserProfile
	err := db.QueryRow("SELECT username, display_name, created_at,elo FROM users WHERE id = ?", userID).Scan(&userResponse.Username, &userResponse.DisplayName, &createdAtFull, &userResponse.Elo)
	if err != nil {
		return userResponse, err
	}
	userResponse.CreatedAt = createdAtFull.Format("2006-01-02")
	if userResponse.DisplayName == "" {
		userResponse.DisplayName = userResponse.Username
	}
	return userResponse, nil
}

func userHandler(db *sql.DB, w http.ResponseWriter, r *http.Request) {
	r.Body = http.MaxBytesReader(w, r.Body, MAX_REQ_SIZE)

	currentUserID := getUserIDFromContext(r)

	targetIDVal := r.PathValue("id")
	targetID, err := strconv.Atoi(targetIDVal)
	if err != nil || targetIDVal == "" {
		targetID = currentUserID
	}

	userResponse, err := getUserProfileByID(db, targetID)
	if err != nil {
		sendErrorResponse(w, "Niepoprawne id użytkownika", http.StatusBadRequest)
		return
	}

	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusOK)

	if currentUserID == targetID {
		json.NewEncoder(w).Encode(userResponse)
	} else {
		simplifiedProfile := userResponse.Simplify()
		json.NewEncoder(w).Encode(simplifiedProfile)
	}
}
